@@ -8,12 +8,16 @@
  */
 
 #include "biascorrection.h"
+#include "scaffolds.h"
 #include "abundances.h"
 #include "progressbar.h"
+#include "bundles.h"
 
 #include <iostream>
 #include <fstream>
 
+using namespace std;
+
 void output_vector(vector<double>& v, char* fname)
 {
 	ofstream myfile1;
@@ -97,7 +101,7 @@ void learn_bias(BundleFactory& bundle_factory, BiasLearner& bl, bool progress_ba
 
 	ProgressBar p_bar;
 	if (progress_bar)
-		ProgressBar p_bar("Learning bias parameters.", bundle_factory.num_bundles());
+		p_bar = ProgressBar("Learning bias parameters.", bundle_factory.read_group_properties()->total_map_mass());
 
 	while(true)
 	{
@@ -114,7 +118,7 @@ void learn_bias(BundleFactory& bundle_factory, BiasLearner& bl, bool progress_ba
 		char bundle_label_buf[2048];
 		sprintf(bundle_label_buf, "%s:%d-%d", rt.get_name(bundle.ref_id()),	bundle.left(), bundle.right());
 		if (progress_bar)
-			p_bar.update(bundle_label_buf, 1);
+			p_bar.update(bundle_label_buf, bundle.raw_mass());
 		
 		if (bundle.non_redundant_hits().size()==0 || bundle.ref_scaffolds().size() != 1)
 		{
@@ -131,103 +135,34 @@ void learn_bias(BundleFactory& bundle_factory, BiasLearner& bl, bool progress_ba
 		p_bar.complete();
 		
 	bl.normalizeParameters();
-#if ADAM_MODE
-	bl.output();
-#endif
+    
+    if (output_bias_params)
+        bl.output();
 }
 
-// void process_bundle(HitBundle& bundle, BiasLearner& bl)
-// {
-// 	const vector<shared_ptr<Scaffold> >& transcripts = bundle.ref_scaffolds();
-// 	const vector<MateHit>& nr_alignments = bundle.non_redundant_hits();
-// 	
-// 	int M = nr_alignments.size();
-// 	int N = transcripts.size();
-// 	
-// 	vector<list<int> > compatibilities(M,list<int>());
-// 	get_compatibility_list(transcripts, nr_alignments, compatibilities);	
-// 	
-// 	vector<vector<double> > startHists(N, vector<double>());
-// 	vector<vector<double> > endHists(N, vector<double>());
-// 	vector<double> useable_mass (N, 0.0);
-// 	
-// 	for (int j = 0; j < N; ++j)
-// 	{
-// 		int size = transcripts[j]->length();
-// 		startHists[j].resize(size+1); // +1 catches overhangs
-// 		endHists[j].resize(size+1);
-// 	}
-// 	
-// 	for (int i = 0; i < M; ++i)
-// 	{
-// 		const MateHit& hit = nr_alignments[i];
-// 		if (!hit.left_alignment() && !hit.right_alignment())
-// 			continue;
-// 
-// 		double num_hits = nr_alignments[i].collapse_mass();
-// 		long double locus_fpkm = 0;
-// 		
-// 		for (list<int>::iterator it=compatibilities[i].begin(); it!=compatibilities[i].end(); ++it)
-// 		{
-// 			locus_fpkm += transcripts[*it]->fpkm(); 
-// 		}
-// 		
-// 		if (locus_fpkm==0)
-// 			continue;
-// 		
-// 		for (list<int>::iterator it=compatibilities[i].begin(); it!=compatibilities[i].end(); ++it)
-// 		{	
-// 			const Scaffold& transcript = *transcripts[*it];
-// 			assert(transcript.strand()!=CUFF_STRAND_UNKNOWN); //Filtered in compatibility list
-// 			
-// 			int start;
-// 			int end;
-// 			int frag_len;
-// 			
-// 			double mass_fraction = num_hits*transcript.fpkm()/locus_fpkm;
-// 			
-// 			transcript.map_frag(hit, start, end, frag_len);
-// 			
-// 			if (start != transcript.length() || end != transcript.length())
-// 				useable_mass[*it] += mass_fraction;
-// 			
-// 			startHists[*it][start] += mass_fraction;
-// 			endHists[*it][end] += mass_fraction;
-// 		}
-// 	}
-//  	for (int j = 0; j < N; ++j)
-//  	{
-//  		if (transcripts[j]->strand()!=CUFF_STRAND_UNKNOWN && 
-// 			transcripts[j]->seq()!="" &&  
-// //			useable_mass[j]/transcripts[j]->length() >= 0.1 &&
-// 			transcripts[j]->fpkm() >= 1)
-//  			bl.processTranscript(startHists[j], endHists[j], *transcripts[j]);
-//  	}
-// }
-
 const int BiasLearner::pow4[] = {1,4,16,64};
 const int BiasLearner::siteSpec[] = {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
 const int BiasLearner::vlmmSpec[] = {1,1,1,1,1,2,2,2,3,3,3,3,3,3,3,3,2,2,2,1,1}; //Length of connections at each position in the window
 const int BiasLearner::MAX_SLICE = 3; // Maximum connection length
 const int BiasLearner::CENTER = 8; //Index in paramTypes[] of first element in read
-const int BiasLearner::_M = 21; //Number of positions spanned by window
-const int BiasLearner::_N = 64; //Length of maximum connection in VLMM
+const int BiasLearner::_m = 21; //Number of positions spanned by window
+const int BiasLearner::_n = 64; //Length of maximum connection in VLMM
 const int BiasLearner::lengthBins[] = {791,1265,1707,2433}; //Quantiles derived from human mRNA length distribution in UCSC genome browser
 const double BiasLearner::positionBins[] = {.02,.04,.06,.08,.10,.15,.2,.3,.4,.5,.6,.7,.8,.85,.9,.92,.94,.96,.98,1};
-//const double BiasLearner::positionBins[] = {0.01,0.02,0.03,0.04,0.05,0.06,0.07,0.08,0.09,0.10,0.11,0.12,0.13,0.14,0.15,0.16,0.17,0.18,0.19,0.20,0.21,0.22,0.23,0.24,0.25,0.26,0.27,0.28,0.29,0.30,0.31,0.32,0.33,0.34,0.35,0.36,0.37,0.38,0.39,0.40,0.41,0.42,0.43,0.44,0.45,0.46,0.47,0.48,0.49,0.50,0.51,0.52,0.53,0.54,0.55,0.56,0.57,0.58,0.59,0.60,0.61,0.62,0.63,0.64,0.65,0.66,0.67,0.68,0.69,0.70,0.71,0.72,0.73,0.74,0.75,0.76,0.77,0.78,0.79,0.80,0.81,0.82,0.83,0.84,0.85,0.86,0.87,0.88,0.89,0.90,0.91,0.92,0.93,0.94,0.95,0.96,0.97,0.98,0.99,1};
 
 BiasLearner::BiasLearner(shared_ptr<EmpDist const> frag_len_dist)
 {
 	paramTypes = vlmmSpec;
-	if (bias_mode=="site")
+	if (bias_mode==SITE || bias_mode==POS_SITE)
 	{
 		paramTypes = siteSpec;
 	}
 	_frag_len_dist = frag_len_dist;
-	_startSeqParams = ublas::zero_matrix<long double>(_M,_N);
-	_startSeqExp = ublas::zero_matrix<long double>(_M,_N);
-	_endSeqParams = ublas::zero_matrix<long double>(_M,_N);
-	_endSeqExp = ublas::zero_matrix<long double>(_M,_N);
+	    
+    _startSeqParams = ublas::zero_matrix<long double>(_m,_n);
+	_startSeqExp = ublas::zero_matrix<long double>(_m,_n);
+	_endSeqParams = ublas::zero_matrix<long double>(_m,_n);
+	_endSeqExp = ublas::zero_matrix<long double>(_m,_n);
 	_startPosParams = ublas::zero_matrix<long double>(20,5);
 	_startPosExp = ublas::zero_matrix<long double>(20,5);
 	_endPosParams = ublas::zero_matrix<long double>(20,5);
@@ -324,7 +259,7 @@ void BiasLearner::processTranscript(const std::vector<double>& startHist, const
 	{
 		
 		//Position Bias
-		if (i > startBinCutoff && currStartBin < _startPosParams.size1()-1)
+		if (i > startBinCutoff && currStartBin < (int)_startPosParams.size1()-1)
 			startBinCutoff=positionBins[++currStartBin]*(seqLen - min_frag_len);
 		if (i - min_frag_len > endBinCutoff)
 			endBinCutoff = positionBins[++currEndBin]*(seqLen - min_frag_len);
@@ -335,14 +270,14 @@ void BiasLearner::processTranscript(const std::vector<double>& startHist, const
 		_endPosExp(currEndBin, lenClass) += !(_frag_len_dist->too_short(i+1));
 
 		
-		bool start_in_bounds = i-CENTER >= 0 && i+(_M-1)-CENTER < seqLen;
-		bool end_in_bounds = i+CENTER-(_M-1) >= 0 && i+CENTER < seqLen;
+		bool start_in_bounds = i-CENTER >= 0 && i+(_m-1)-CENTER < seqLen;
+		bool end_in_bounds = i+CENTER-(_m-1) >= 0 && i+CENTER < seqLen;
 		
 		if (!start_in_bounds && !end_in_bounds) // Make sure we are in bounds of the sequence
 			continue;
 		
 		//Sequence Bias
-		for(int j=0; j < _M; j++)
+		for(int j=0; j < _m; j++)
 		{
 			// Start Bias
 			if (start_in_bounds) // Make sure we are in bounds of the sequence
@@ -420,7 +355,7 @@ void BiasLearner::getBias(const Scaffold& transcript, vector<double>& startBiase
 	for (int i=0; i < seqLen; i++)
 	{
 		//Position Bias
-		if (i > startBinCutoff && currStartBin < _startPosParams.size1()-1)
+		if (i > startBinCutoff && currStartBin < (int)_startPosParams.size1()-1)
 			startBinCutoff=positionBins[++currStartBin]*(seqLen - min_frag_len);
 		if (i - min_frag_len > endBinCutoff)
 			endBinCutoff = positionBins[++currEndBin]*(seqLen - min_frag_len);
@@ -430,12 +365,12 @@ void BiasLearner::getBias(const Scaffold& transcript, vector<double>& startBiase
 		
 		//Sequence Bias
 		
-		bool start_in_bounds = i-CENTER >= 0 && i+(_M-1)-CENTER < seqLen;
-		bool end_in_bounds = i+CENTER-(_M-1) >= 0 && i+CENTER < seqLen - _frag_len_dist->mean(); // don't count bias near end since we're over-counting these fragments
+		bool start_in_bounds = i-CENTER >= 0 && i+(_m-1)-CENTER < seqLen;
+		bool end_in_bounds = i+CENTER-(_m-1) >= 0 && i+CENTER < seqLen - _frag_len_dist->mean(); // don't count bias near end since we're over-counting these fragments
 		
 		if (start_in_bounds || end_in_bounds) // Make sure we are in bounds of the sequence
 		{
-			for(int j=0; j < _M; j++)
+			for(int j=0; j < _m; j++)
 			{
 				// Start Bias
 				if (start_in_bounds) // Make sure we are in bounds of the sequence
@@ -485,6 +420,7 @@ void BiasLearner::getBias(const Scaffold& transcript, vector<double>& startBiase
 				}
 			}
 		}
+        assert(finite(startBias) && finite(endBias));
 		startBiases[i] = startBias;
 		endBiases[i] = endBias;
 	}
@@ -525,6 +461,9 @@ void BiasLearner::genNList(const char* seqSlice, int start, int n, list<int>& nL
 
 void BiasLearner::normalizeParameters()
 {
+    double THRESH = 100;
+
+    
 	//Normalize position parameters	
 	vector<long double> startPosParam_sums;
 	vector<long double> startPosExp_sums;
@@ -540,19 +479,33 @@ void BiasLearner::normalizeParameters()
 	{
 		for(size_t j=0; j < _startPosParams.size2(); j++)
 		{
-			_startPosParams(i,j) /= startPosParam_sums[j];
-			_startPosExp(i,j) /= startPosExp_sums[j];
-			if (_startPosExp(i,j) == 0)
-				_startPosParams(i,j) = numeric_limits<long double>::max();
-			else
-				_startPosParams(i,j) /= _startPosExp(i,j);
+            if (startPosParam_sums[j] < THRESH)
+			{
+				_startPosParams(i,j) = 1;
+			}
+            else
+            {
+                _startPosParams(i,j) /= startPosParam_sums[j];
+                _startPosExp(i,j) /= startPosExp_sums[j];
+                if (_startPosExp(i,j) == 0)
+                    _startPosParams(i,j) = numeric_limits<long double>::max();
+                else
+                    _startPosParams(i,j) /= _startPosExp(i,j);
+            }
 			
-			_endPosParams(i,j) /= endPosParam_sums[j];
-			_endPosExp(i,j) /= endPosExp_sums[j];
-			if (_endPosExp(i,j) == 0)
-				_endPosParams(i,j) = numeric_limits<long double>::max();
-			else
-				_endPosParams(i,j) /= _endPosExp(i,j);
+            if (endPosParam_sums[j] < THRESH)
+			{
+				_endPosParams(i,j) = 1;
+			}
+            else
+            {
+                _endPosParams(i,j) /= endPosParam_sums[j];
+                _endPosExp(i,j) /= endPosExp_sums[j];
+                if (_endPosExp(i,j) == 0)
+                    _endPosParams(i,j) = numeric_limits<long double>::max();
+                else
+                    _endPosParams(i,j) /= _endPosExp(i,j);
+            }
 		}
 	}
 	
@@ -570,13 +523,13 @@ void BiasLearner::normalizeParameters()
 	ublas::matrix<long double> endParam_sums;
 	end_tot = fourSums(_endSeqParams, endParam_sums);
 	fourSums(_endSeqExp, endSeqExp_sums); 
-	
+	    
 	//Normalize sequence parameters
-	for(int i=0; i < _M; i++)
+	for(int i=0; i < _m; i++)
 	{
 		for(int j=0; j < pow4[paramTypes[i]]; j++)
 		{
-			if (startParam_sums(i,j/4)==0)
+			if (startParam_sums(i,j/4) < THRESH)
 			{
 				_startSeqParams(i,j) = 1;
 			}
@@ -586,7 +539,7 @@ void BiasLearner::normalizeParameters()
 				_startSeqExp(i,j) /= startSeqExp_sums(i,j/4);
 				_startSeqParams(i,j) /= _startSeqExp(i,j);
 			}
-			if (endParam_sums(i,j/4)==0)
+			if (endParam_sums(i,j/4) < THRESH)
 			{
 				_endSeqParams(i,j) = 1;
 			}
@@ -596,7 +549,6 @@ void BiasLearner::normalizeParameters()
 				_endSeqExp(i,j) /= endSeqExp_sums(i,j/4);
 				_endSeqParams(i,j) /= _endSeqExp(i,j);
 			}
-
 		}
 	}
 	
@@ -605,12 +557,12 @@ void BiasLearner::normalizeParameters()
 	if (end_tot==0.0)
 		ones(_endSeqParams);
 	
-	if (bias_mode=="vlmm" || bias_mode=="site")
+	if (bias_mode==VLMM || bias_mode==SITE)
 	{
 		ones(_startPosParams);
 		ones(_endPosParams);
 	}
-	else if (bias_mode =="pos")	
+	else if (bias_mode == POS)	
 	{
 		ones(_startSeqParams);
 		ones(_endSeqParams);
@@ -624,18 +576,18 @@ void BiasLearner::output()
 	myfile1.open (filename.c_str());
 	
 	// StartSeq
-	for (int i = 0; i < _N; ++i)
+	for (int i = 0; i < _n; ++i)
 	{
-		for(int j = 0; j < _M; ++j)
+		for(int j = 0; j < _m; ++j)
 			myfile1 << _startSeqParams(j,i) <<",";
 		myfile1 << endl;
 	}
 	myfile1 << endl;
 	
 	// EndSeq
-	for (int i = 0; i < _N; ++i)
+	for (int i = 0; i < _n; ++i)
 	{
-		for(int j = 0; j < _M; ++j)
+		for(int j = 0; j < _m; ++j)
 			myfile1 << _endSeqParams(j,i) <<",";
 		myfile1 << endl;
 	}
@@ -673,12 +625,7 @@ int BiasCorrectionHelper::add_read_group(shared_ptr<ReadGroupProperties const> r
 	vector<double> end_bias(trans_len+1, 1.0);
 	double eff_len = 0.0;
 	
-	if (rgp->bias_learner()!=NULL && _transcript->strand()!=CUFF_STRAND_UNKNOWN)
-	{
-		rgp->bias_learner()->getBias(*_transcript, start_bias, end_bias);
-	}
-	
-	shared_ptr<EmpDist const> frag_len_dist = rgp->frag_len_dist();
+	shared_ptr<EmpDist const> fld = rgp->frag_len_dist();
 	
 	vector<double> tot_bias_for_len(trans_len+1, 0);
 	vector<double> start_bias_for_len(trans_len+1, 0);
@@ -688,27 +635,36 @@ int BiasCorrectionHelper::add_read_group(shared_ptr<ReadGroupProperties const> r
 	start_bias_for_len[trans_len] = trans_len;
 	end_bias_for_len[trans_len] = trans_len;
 
-	
-	for(int l = rgp->frag_len_dist()->min(); l <= trans_len; l++)
+	if (final_est_run && corr_bias && _transcript->strand()!=CUFF_STRAND_UNKNOWN)
 	{
-		//double tot = 0;
-		//double start = 0;
-		//double end = 0;
-		for(int i = 0; i <= trans_len - l; i++)
+		rgp->bias_learner()->getBias(*_transcript, start_bias, end_bias);
+
+		for(int l = fld->min(); l <= trans_len; l++)
 		{
-			double tot_bias = start_bias[i]*end_bias[i+l-1];
-			tot_bias_for_len[l] += tot_bias;
-			start_bias_for_len[l] += start_bias[i];
-			end_bias_for_len[l] += end_bias[i+l-1];
-			eff_len += tot_bias * frag_len_dist->npdf(l, trans_len - i);
+			for(int i = 0; i <= trans_len - l; i++)
+			{
+				double tot_bias = start_bias[i]*end_bias[i+l-1];
+				tot_bias_for_len[l] += tot_bias;
+				start_bias_for_len[l] += start_bias[i];
+				end_bias_for_len[l] += end_bias[i+l-1];
+				
+				double frag_prob = (bias_mode == POS || bias_mode == POS_VLMM || bias_mode == POS_SITE) ? fld->npdf(l, trans_len-i) : fld->pdf(l);
+				eff_len += tot_bias * frag_prob;
+			}
+		}
+	}
+	else
+	{
+		for(int l = fld->min(); l <= trans_len; l++)
+		{
+			tot_bias_for_len[l] = trans_len - l + 1;
+			start_bias_for_len[l] = trans_len - l + 1;
+			end_bias_for_len[l] = trans_len - l + 1;
+			eff_len += fld->pdf(l) * (trans_len - l + 1);
 		}
-        //assert(tot != 0);
-		//tot_bias_for_len[l] = tot;
-		//eff_len += tot * p_len;
-		//mean_start_bias += start * p_len;
-		//mean_end_bias += end * p_len; 
 	}
 	
+	assert(eff_len > 0);
 	_start_biases.push_back(start_bias);
 	_end_biases.push_back(end_bias);
 	_tot_biases_for_len.push_back(tot_bias_for_len);
@@ -735,7 +691,7 @@ int BiasCorrectionHelper::get_index(shared_ptr<ReadGroupProperties const> rgp)
 	return iter->second;
 }
 
-// Hit needs to be collapsed
+// Hit needs to be from the collapsed (non_redundant) list to match indexing 
 double BiasCorrectionHelper::get_cond_prob(const MateHit& hit)
 {
 	shared_ptr<ReadGroupProperties const> rgp = hit.read_group_props();
@@ -754,30 +710,90 @@ double BiasCorrectionHelper::get_cond_prob(const MateHit& hit)
 	double cond_prob = 1.0;
 	cond_prob *= _start_biases[i][start];
 	cond_prob *= _end_biases[i][end];
-	cond_prob *= fld->npdf(frag_len, trans_len-start); //defaults to pdf if trans_len==start
+	double frag_prob = (bias_mode == POS || bias_mode == POS_VLMM || bias_mode == POS_SITE) ? fld->npdf(frag_len, trans_len-start) : fld->pdf(frag_len);
+	cond_prob *= frag_prob; 
 	
 	if (cond_prob==0.0)
 		return 0.0;
 	
-	if (hit.is_pair())
-		cond_prob /= _tot_biases_for_len[i][frag_len];
+    if (hit.is_pair() || hit.read_group_props()->complete_fragments())
+    {
+        if (frag_len >= (int)_tot_biases_for_len[i].size())
+            cond_prob = 0.0;
+        else
+            cond_prob /= _tot_biases_for_len[i][frag_len];
+    }
 	else if (start!=trans_len && end==trans_len) // The hit is a singleton at the start of a fragment
 		cond_prob /= _start_biases_for_len[i][frag_len];
 	else if (start==trans_len && end!=trans_len) // The hit is a singleton at the end of a fragment
 		cond_prob /= _end_biases_for_len[i][frag_len];
 	else if (frag_len==trans_len)  // We don't actually know where we start or end and can't subtract off the frag_len or we'll get inf
 		cond_prob /= trans_len;
-	else // Single-end read w/ library type FF or RR
-		cond_prob /= trans_len-frag_len;
+	else
+    {
+        if (trans_len < frag_len)
+        {
+            cond_prob = 0;
+        }
+        else 
+        {
+            // Single-end read w/ library type FF or RR
+            cond_prob /= trans_len-frag_len;
+        }
+    }
 
 	if (cond_prob > 0 && hit.collapse_mass() > 0)
 	{
-		_rg_masses[i] += hit.collapse_mass();
+		_rg_masses[i] +=  hit.collapse_mass();
 		_mapped = true;
 	}
 	
+#if DEBUG
+    if (isinf(cond_prob))
+    {
+        double cond_prob = 1.0;
+        cond_prob *= _start_biases[i][start];
+        cond_prob *= _end_biases[i][end];
+        double frag_prob = (bias_mode == POS || bias_mode == POS_VLMM || bias_mode == POS_SITE) ? fld->npdf(frag_len, trans_len-start) : fld->pdf(frag_len);
+        cond_prob *= frag_prob; 
+        
+        if (cond_prob==0.0)
+            return 0.0;
+        
+        if (hit.is_pair())
+        {
+            if (frag_len >= _tot_biases_for_len[i].size())
+                cond_prob = 0.0;
+            else
+                cond_prob /= _tot_biases_for_len[i][frag_len];
+        }
+        else if (start!=trans_len && end==trans_len) // The hit is a singleton at the start of a fragment
+            cond_prob /= _start_biases_for_len[i][frag_len];
+        else if (start==trans_len && end!=trans_len) // The hit is a singleton at the end of a fragment
+            cond_prob /= _end_biases_for_len[i][frag_len];
+        else if (frag_len==trans_len)  // We don't actually know where we start or end and can't subtract off the frag_len or we'll get inf
+            cond_prob /= trans_len;
+        else
+        {
+            if (trans_len < frag_len)
+            {
+                cond_prob = 0;
+            }
+            else 
+            {
+                // Single-end read w/ library type FF or RR
+                cond_prob /= trans_len-frag_len;
+            }
+        }
+    }
+#endif
+    
 	assert(!isinf(cond_prob));
 	assert(!isnan(cond_prob));
+    
+    if (isinf(cond_prob) || isnan(cond_prob))
+        cond_prob = 0.0;
+    
 	return cond_prob;
 }
 
@@ -804,6 +820,7 @@ double BiasCorrectionHelper::get_effective_length()
 	}
 	
 	assert(eff_len>0);
+    //assert(eff_len>1);
 	assert(!isnan(eff_len));
 	return eff_len;
 }
