@@ -18,7 +18,9 @@
 #include "genes.h"
 
 void filter_junk_isoforms(vector<shared_ptr<Abundance> >& transcripts,
-						  vector<double>& abundances);
+						  vector<double>& abundances,
+                          const vector<shared_ptr<Abundance> >& mapped_transcripts,
+                          double locus_mass);
 
 
 void filter_introns(int bundle_length,
@@ -35,4 +37,6 @@ void filter_junk_genes(vector<Gene>& genes);
 
 void filter_hits(int bundle_length, int bundle_left, vector<Scaffold>& hits);
 
+void clip_by_3_prime_dropoff(vector<Scaffold>& scaff);
+
 #endif
