@@ -105,50 +105,33 @@ void collect_non_redundant_ops(const vector<Scaffold>& scaffolds,
 	sort (ops.begin(), ops.end(), op_left_lt_right_lt);   
 }
 
-void fill_unambiguous_unknowns(vector<Scaffold>& scaffolds)
+void fill_unambiguous_unknowns(vector<Scaffold>& to_fill, 
+                               const vector<Scaffold>& constitutive)
 {
-    vector<AugmentedCuffOp> conflict_ops;
-    vector<AugmentedCuffOp> ops;
-    
-    collect_non_redundant_ops(scaffolds, ops);
-    
-    extract_conflicting_ops(ops, conflict_ops);
-    
-    sort(conflict_ops.begin(), conflict_ops.end());
-    sort(ops.begin(), ops.end());
-    
-    vector<AugmentedCuffOp> non_conflict;
-    
-    set_difference(ops.begin(), 
-                   ops.end(), 
-                   conflict_ops.begin(), 
-                   conflict_ops.end(), 
-                   back_inserter(non_conflict));
-    sort(non_conflict.begin(), non_conflict.end(), AugmentedCuffOp::g_left_lt);
-    vector<AugmentedCuffOp> merged;
-    
-    CuffStrand s = CUFF_STRAND_UNKNOWN;
-    for (size_t i = 0; i < scaffolds.size(); ++i)
-    {
-        if (scaffolds[i].strand() != CUFF_STRAND_UNKNOWN)
-        {
-            if (s != CUFF_STRAND_UNKNOWN)
-            {
-                assert (s == scaffolds[i].strand());
-            }
-            else 
-            {
-                s = scaffolds[i].strand();
-            }
+//    vector<AugmentedCuffOp> conflict_ops;
+//    vector<AugmentedCuffOp> ops;
 
-        }
-    }
-    
-    AugmentedCuffOp::merge_ops(non_conflict, merged, true);
-    for (size_t i = 0; i < scaffolds.size(); ++i)
+    for (size_t i = 0; i < to_fill.size(); ++i)
     {
-        scaffolds[i].strand(s);
-        scaffolds[i].fill_gaps(merged);
+        if (to_fill[i].has_unknown())
+        {
+            for( size_t j = 0; j < constitutive.size(); ++j)
+            {
+                const Scaffold& cons = constitutive[j];
+                if (Scaffold::overlap_in_genome(to_fill[i], cons, 0) &&
+                    Scaffold::compatible(to_fill[i], cons))
+                {
+                    if (cons.strand() != CUFF_STRAND_UNKNOWN)
+                        to_fill[i].strand(cons.strand());
+
+                    to_fill[i].fill_gaps(cons.augmented_ops());
+                    if (!to_fill[i].has_unknown())
+                    {
+                        break;
+                    }
+                }
+            }
+        }
     }
 }
 
@@ -198,7 +181,7 @@ void add_non_constitutive_to_scaffold_mask(const vector<Scaffold>& scaffolds,
 		}
 	}
     
-	asm_verbose("%lu constitutive reads of %lu smash-filtered from further consideration\n", num_filtered, smash_filter.size());
+	verbose_msg("%lu constitutive reads of %lu smash-filtered from further consideration\n", num_filtered, smash_filter.size());
     
     vector<AugmentedCuffOp> ops;
     collect_non_redundant_ops(scaffolds, ops);
@@ -239,7 +222,7 @@ bool collapse_contained_transfrags(vector<Scaffold>& scaffolds,
 	while (max_rounds--)
 	{
 		
-		asm_verbose("%s\tStarting new collapse round\n", bundle_label->c_str());
+		verbose_msg("%s\tStarting new collapse round\n", bundle_label->c_str());
         
 		ContainmentGraph containment;
 		
@@ -290,12 +273,12 @@ bool collapse_contained_transfrags(vector<Scaffold>& scaffolds,
 		
 		lemon::MaxBipartiteMatching<ContainmentGraph>  matcher(containment);
         
-		asm_verbose("%s\tContainment graph has %d nodes, %d edges\n", bundle_label->c_str(), containment.aNodeNum(), containment.uEdgeNum());
-		asm_verbose("%s\tFinding a maximum matching to collapse scaffolds\n", bundle_label->c_str());
+		verbose_msg("%s\tContainment graph has %d nodes, %d edges\n", bundle_label->c_str(), containment.aNodeNum(), containment.uEdgeNum());
+		verbose_msg("%s\tFinding a maximum matching to collapse scaffolds\n", bundle_label->c_str());
         
 		matcher.run();
         
-		asm_verbose( "%s\tWill collapse %d scaffolds\n", bundle_label->c_str(), matcher.matchingSize());
+		verbose_msg( "%s\tWill collapse %d scaffolds\n", bundle_label->c_str(), matcher.matchingSize());
 		
 		ContainmentGraph::UEdgeMap<bool> matched_edges(containment);
 		
@@ -389,12 +372,12 @@ bool collapse_equivalent_transfrags(vector<Scaffold>& fragments,
 			 smaller_idx_array.end(), 
 			 FragIndexSortSmallerLR(fragments));
 		
-		asm_verbose("%s\tStarting new collapse round\n", bundle_label->c_str());
-        asm_verbose("%s\tFinding fragment-level conflicts\n", bundle_label->c_str());
+		verbose_msg("%s\tStarting new collapse round\n", bundle_label->c_str());
+        verbose_msg("%s\tFinding fragment-level conflicts\n", bundle_label->c_str());
 
         bool will_perform_collapse = false;
 		
-        asm_verbose( "%s\tAssessing overlaps between %lu fragments for identical conflict sets\n", 
+        verbose_msg( "%s\tAssessing overlaps between %lu fragments for identical conflict sets\n", 
                 bundle_label->c_str(), 
                 fragments.size());
         vector<size_t> replacements;
@@ -452,21 +435,24 @@ bool collapse_equivalent_transfrags(vector<Scaffold>& fragments,
 				
 				double lhs_len = lhs_scaff.right() - lhs_scaff.left();
 				
-				for (int i = 0; i < smaller_idx_array.size(); ++i)
+				for (size_t i = 0; i < smaller_idx_array.size(); ++i)
 				{
 					size_t j_scaff_idx = smaller_idx_array[i];
-					if (Scaffold::overlap_in_genome(lhs_scaff, fragments[j_scaff_idx], 0))
-					{
-						if (!Scaffold::compatible(lhs_scaff, fragments[j_scaff_idx]))
-						{
-							curr_conflicts.push_back(j_scaff_idx);
-						}
-					}
+                    if (replacements[j_scaff_idx] == j_scaff_idx)
+                    {
+                        if (Scaffold::overlap_in_genome(lhs_scaff, fragments[j_scaff_idx], 0))
+                        {
+                            if (!Scaffold::compatible(lhs_scaff, fragments[j_scaff_idx]))
+                            {
+                                curr_conflicts.push_back(j_scaff_idx);
+                            }
+                        }
+                    }
 				}
 				sort(curr_conflicts.begin(), curr_conflicts.end());
 				
 				//bool advanced_curr = false;
-				for (int c = lhs + 1; c < smaller_idx_array.size(); ++c)
+				for (size_t c = lhs + 1; c < smaller_idx_array.size(); ++c)
 				{
 					size_t c_native_idx = smaller_idx_array[c];
 					const Scaffold& c_scaff = fragments[c_native_idx];
@@ -482,7 +468,7 @@ bool collapse_equivalent_transfrags(vector<Scaffold>& fragments,
 						{
 							if (num_merges % 100 == 0)
 							{
-								asm_verbose("%s\tCollapsing frag # %d\n", 
+								verbose_msg("%s\tCollapsing frag # %d\n", 
 										bundle_label->c_str(), 
 										num_merges);
 							}
@@ -527,25 +513,30 @@ bool collapse_equivalent_transfrags(vector<Scaffold>& fragments,
 					
 						// Now check that c doesn't have any additional conflicts
 						// of it's own
-						for (int i = lhs_native_idx + 1; i < fragments.size(); ++i)
+						for (size_t i = lhs_native_idx + 1; i < fragments.size(); ++i)
 						{
-							if (Scaffold::overlap_in_genome(fragments[i], lhs_scaff, 0))
-							{
-								if (Scaffold::overlap_in_genome(fragments[i], c_scaff, 0) &&
-									!Scaffold::compatible(fragments[i], c_scaff))
-								{
-									//c_conflicts.push_back(i);
-									if (!binary_search(curr_conflicts.begin(), curr_conflicts.end(), i))
-									{
-										not_equivalent = true;
-										break;
-									}
-								}
-							}
-							else
-							{
-								break;
-							}
+                            if (replacements[i] == i)
+                            {
+                                if (Scaffold::overlap_in_genome(fragments[i], lhs_scaff, 0))
+                                {
+                                    if (Scaffold::overlap_in_genome(fragments[i], c_scaff, 0))
+                                    {
+                                        if (!Scaffold::compatible(fragments[i], c_scaff))
+                                        {
+                                            //c_conflicts.push_back(i);
+                                            if (!binary_search(curr_conflicts.begin(), curr_conflicts.end(), i))
+                                            {
+                                                not_equivalent = true;
+                                                break;
+                                            }
+                                        }
+                                    }
+                                }
+                                else
+                                {
+                                    break;
+                                }
+                            }
 						}
 						
 						
@@ -555,7 +546,7 @@ bool collapse_equivalent_transfrags(vector<Scaffold>& fragments,
 						// merge
 						if (num_merges % 100 == 0)
 						{
-							asm_verbose("%s\tCollapsing frag # %d\n", 
+							verbose_msg("%s\tCollapsing frag # %d\n", 
 									bundle_label->c_str(), 
 									num_merges);
 						}
@@ -611,7 +602,7 @@ void compress_consitutive(vector<Scaffold>& hits)
 {
     vector<bool> scaffold_mask;
     
-    asm_verbose("%s\tBuilding constitutivity mask\n", bundle_label->c_str()); 
+    verbose_msg("%s\tBuilding constitutivity mask\n", bundle_label->c_str()); 
     
     scaffold_mask = vector<bool>(hits.size(), false);
     add_non_constitutive_to_scaffold_mask(hits, scaffold_mask);
@@ -639,12 +630,16 @@ void compress_consitutive(vector<Scaffold>& hits)
         hits.insert(hits.end(), completes.begin(), completes.end()); 
     }
     
+    fill_unambiguous_unknowns(non_constitutive, hits);
+    
     hits.insert(hits.end(), non_constitutive.begin(), non_constitutive.end());
     sort(hits.begin(), hits.end(), scaff_lt);
+    
+    
     size_t post_compress = hits.size();
     size_t delta = pre_compress - post_compress;
     double collapse_ratio = delta / (double) pre_compress; 
-    asm_verbose("%s\tCompressed %lu of %lu constitutive fragments (%lf percent)\n", 
+    verbose_msg("%s\tCompressed %lu of %lu constitutive fragments (%lf percent)\n", 
             bundle_label->c_str(),
             delta, 
             pre_compress, 
@@ -665,6 +660,8 @@ void compress_redundant(vector<Scaffold>& fragments)
         rightmost = std::max((long)fragments[i].right(), rightmost);
     }
     
+    size_t pre_compress = fragments.size();
+    
     while (true)
     {
         if (last_size == -1 || 0.9 * last_size > fragments.size())
@@ -680,22 +677,29 @@ void compress_redundant(vector<Scaffold>& fragments)
             break;
         }
     }
+    
+    size_t post_compress = fragments.size();
+    size_t delta = pre_compress - post_compress;
+    double collapse_ratio = delta / (double) pre_compress; 
+    verbose_msg("%s\tCompressed %lu of %lu redundant fragments (%lf percent)\n", 
+                bundle_label->c_str(),
+                delta, 
+                pre_compress, 
+                collapse_ratio);
 }
 
 void compress_fragments(vector<Scaffold>& fragments)
 {
-    asm_verbose("%s\tPerforming preliminary containment collapse on %lu fragments\n", bundle_label->c_str(), fragments.size());
+    verbose_msg("%s\tPerforming preliminary containment collapse on %lu fragments\n", bundle_label->c_str(), fragments.size());
     size_t pre_hit_collapse_size = fragments.size();
     sort(fragments.begin(), fragments.end(), scaff_lt_rt);
-	
-    fill_unambiguous_unknowns(fragments);
     
 	compress_consitutive(fragments);
 	
 	compress_redundant(fragments);
     
     size_t post_hit_collapse_size = fragments.size();
-    asm_verbose("%s\tIgnoring %lu strictly contained fragments\n", bundle_label->c_str(), pre_hit_collapse_size - post_hit_collapse_size);
+    verbose_msg("%s\tIgnoring %lu strictly contained fragments\n", bundle_label->c_str(), pre_hit_collapse_size - post_hit_collapse_size);
 }
 
 void compress_overlap_dag_paths(DAG& bundle_dag,
@@ -728,13 +732,13 @@ void compress_overlap_dag_paths(DAG& bundle_dag,
         if (!compressed_paths[i].empty())
         {
             Scaffold s(compressed_paths[i]);
-            asm_verbose("Path over %d-%d has %lu fragments in it\n", s.left(), s.right(), compressed_paths[i].size());
+            verbose_msg("Path over %d-%d has %lu fragments in it\n", s.left(), s.right(), compressed_paths[i].size());
             new_scaffs.push_back(s);
         }
     }
     //hits = new_scaffs;
 	
-    asm_verbose("%s\tCompressed overlap graph from %lu to %lu fragments (%f percent)\n",
+    verbose_msg("%s\tCompressed overlap graph from %lu to %lu fragments (%f percent)\n",
             bundle_label->c_str(), 
             hits.size(), 
             new_scaffs.size(), 
