@@ -16,7 +16,6 @@
 
 
 #define MAX_QFILES 500
-extern const char* ATTR_GENE_NAME;
 
 extern bool gtf_tracking_verbose;
 
@@ -26,6 +25,9 @@ extern bool gtf_tracking_largeScale;
 
 int cmpByPtr(const pointer p1, const pointer p2);
 
+bool t_contains(GffObj& a, GffObj& b); 
+//returns true only IF b has fewer exons than a AND a "contains" b
+
 char* getGSeqName(int gseq_id);
 
 //genomic fasta sequence handling
@@ -74,7 +76,7 @@ class GFastaHandler {
             faIdx=new GFastaIndex(fastaPath,fainame.chars());
             GStr fainamecwd(fainame);
             int ip=-1;
-            if ((ip=fainamecwd.rindex(CHPATHSEP))>=0)
+            if ((ip=fainamecwd.rindex('/'))>=0)
                fainamecwd.cut(0,ip+1);
             if (!faIdx->hasIndex()) { //could not load index
                //try current directory
@@ -139,26 +141,30 @@ class GFastaHandler {
      }
 };
 
+
+
+bool betterRef(GffObj* a, GffObj* b); //for better CovLink reference ranking
+
 class GLocus;
 
 class COvLink {
 public:
-    static int coderank(char c) {
+	static int coderank(char c) {
 		switch (c) {
 			case '=': return 0; //ichain match
-			case 'c': return 2; //containment (chain fragment)
-			case 'e': return 4; // overlap exon+intron of unspliced pre-mRNA
-			case 'j': return 6; // overlap with at least a junction match
-			case 'o': return 8; // overlap (exon)
-			case 'r': return 14; //repeats
-			case 'i': return 16; // intra-intron
-            case 'x': return 18; // plain exon overlap on opposite strand
-			case 'p': return 20; //polymerase run
-			case 'u': return 90; //intergenic
-            case 's': return 94; //"shadow" - at least one (fuzzy) intron overlap on the opposite strand
+			case 'c': return 2; //containment (ichain fragment)
+			case 'j': return 4; // overlap with at least a junction match
+			case 'e': return 6; // single exon transfrag overlapping an intron of reference (possible pre-mRNA)
+			case 'o': return 8; // generic exon overlap
+			case 's': return 16; //"shadow" - an intron overlaps with a ref intron on the opposite strand
+			case 'x': return 18; // exon overlap on opposite strand (usually wrong strand mapping)
+			case 'i': return 20; // intra-intron
+			case 'p': return 90; //polymerase run
+			case 'r': return 92; //repeats
+			case 'u': return 94; //intergenic
 			case  0 : return 100;
 			default: return 96;
-        }
+			}
 	}
     char code;
     int rank;
@@ -172,12 +178,12 @@ public:
 	}
     bool operator<(COvLink& b) {
 		if (rank==b.rank)
-			return (ovlen==b.ovlen)? (mrna>b.mrna) : (ovlen>b.ovlen);
+			return (ovlen==b.ovlen)? betterRef(mrna, b.mrna) : (ovlen>b.ovlen);
 		else return rank<b.rank;
 	}
     bool operator>(COvLink& b) {
 		if (rank==b.rank)
-			return (ovlen==b.ovlen)? (mrna<b.mrna) : (ovlen<b.ovlen);
+			return (ovlen==b.ovlen)? betterRef(b.mrna, mrna) : (ovlen<b.ovlen);
 		else return rank>b.rank;
 	}
     bool operator==(COvLink& b) {
@@ -213,16 +219,25 @@ class GIArray:public GArray<GISeg> {
 
 };
 
+class CEqList: public GList<GffObj> {
+  public:
+    GffObj* head;
+    CEqList():GList<GffObj>((GCompareProc*)cmpByPtr, (GFreeProc*)NULL, true) {
+      head=NULL;
+      }
+};
+
 class CTData { //transcript associated data
 public:
-	GffObj* mrna;
+	GffObj* mrna; //owner transcript
 	GLocus* locus;
 	GList<COvLink> ovls; //overlaps with other transcripts (ref vs query)
 	//-- just for ichain match tracking:
-	GffObj* eqref; //ref having an ichain match
-	int qset; //qry set index (qfidx)
-	GffObj* eqnext;
-	int eqdata;
+	GffObj* eqref; //ref transcript having an ichain match
+	int qset; //qry set index (qfidx), -1 means reference dataset
+	//GffObj* eqnext; //next GffObj in the linked list of matching transfrags
+	CEqList* eqlist; //keep track of matching transfrags
+	//int eqdata; // flags for EQ list (is it a list head?)
 	// Cufflinks specific data:
 	double FPKM;
 	double conf_hi;
@@ -233,19 +248,89 @@ public:
 		mrna=m;
 		if (mrna!=NULL) mrna->uptr=this;
 		locus=l;
-        classcode=0;
+		classcode=0;
 		eqref=NULL;
-		eqnext=NULL;
-		eqdata=0;
+		//eqnext=NULL;
+		eqlist=NULL;
+		//eqdata=0;
 		qset=-2;
 		FPKM=0;
 		conf_lo=0;
 		conf_hi=0;
 		cov=0;
 	}
+
 	~CTData() {
 		ovls.Clear();
+		//if ((eqdata & EQHEAD_TAG)!=0) delete eqlist;
+		if (isEqHead()) delete eqlist;
 	}
+
+  //inline bool eqHead() { return ((eqdata & EQHEAD_TAG)!=0); }
+  bool isEqHead() { 
+      if (eqlist==NULL) return false;
+      return (eqlist->head==this->mrna);
+      }
+    
+  void joinEqList(GffObj* m) { //add list from m
+   //list head is set to the transfrag with the lower qset#
+  CTData* md=(CTData*)(m->uptr);
+  //ASSERT(md);
+  if (eqlist==NULL) {
+     if (md->eqlist!=NULL) {
+          eqlist=md->eqlist;
+          eqlist->Add(this->mrna);
+          CTData* md_head_d=(CTData*)(md->eqlist->head->uptr);
+          if (this->qset < md_head_d->qset)
+               eqlist->head=this->mrna;
+          }
+        else { //m was not in an EQ list
+          //eqlist=new GList<GffObj>((GCompareProc*)cmpByPtr, (GFreeProc*)NULL, true);
+          eqlist=new CEqList();
+          eqlist->Add(this->mrna);
+          eqlist->Add(m);
+          md->eqlist=eqlist;
+          if (qset<md->qset) eqlist->head=this->mrna;
+                       else  eqlist->head=m;
+          }
+      }//no eqlist before
+     else { //merge two eqlists
+      if (eqlist==md->eqlist) //already in the same eqlist, nothing to do
+         return;
+      if (md->eqlist!=NULL) { //copy elements of m's eqlist
+        //copy the smaller list into the larger one
+        CEqList* srclst, *destlst;
+        if (md->eqlist->Count()<eqlist->Count()) {
+           srclst=md->eqlist;
+           destlst=eqlist;
+           }
+         else {
+           srclst=eqlist;
+           destlst=md->eqlist;
+           }
+         for (int i=0;i<srclst->Count();i++) {
+           destlst->Add(srclst->Get(i));
+           CTData* od=(CTData*)((*srclst)[i]->uptr);
+           od->eqlist=destlst;
+           //od->eqdata=od->qset+1;
+           }
+        this->eqlist=destlst;
+        CTData* s_head_d=(CTData*)(srclst->head->uptr);
+        CTData* d_head_d=(CTData*)(destlst->head->uptr);
+        if (s_head_d->qset < d_head_d->qset )
+             this->eqlist->head=srclst->head; 
+        delete srclst;
+        }
+       else { //md->eqlist==NULL
+        eqlist->Add(m);
+        md->eqlist=eqlist;
+        CTData* head_d=(CTData*)(eqlist->head->uptr);
+        if (md->qset<head_d->qset)
+            eqlist->head=m;
+        }
+      }
+  }
+
 	void addOvl(char code,GffObj* target=NULL, int ovlen=0) {
 		ovls.AddIfNew(new COvLink(code, target, ovlen));
 	}
@@ -549,7 +634,9 @@ public:
     int total_rintrons; //unique introns
 	
     //--- accuracy data after compared to ref loci:
-	int locusTP; // 1 if mrnaTP>0
+  int locusQTP;
+	int locusTP;
+  int locusAQTP;
 	int locusATP; // 1 if ichainATP + mrnaATP > 0
 	int locusFP;
 	int locusAFP;
@@ -612,7 +699,7 @@ public:
 		qbases_all=0;
 		rbases_all=0;
 		baseTP=0;baseFP=0;baseFN=0;
-		locusTP=0;locusATP=0;
+		locusTP=0;locusQTP=0; locusAQTP=0; locusATP=0;
 		locusFP=0;locusAFP=0;locusAFN=0;
 		locusFN=0;
 		in_rmrnas=0;
@@ -698,8 +785,8 @@ public:
 		mrnaAFP=total_qmrnas-mrnaATP;
 		mrnaAFN=total_rmrnas-mrnaATP;
 		// locus/gene level:
-		locusAFP=total_qloci-locusATP;
-		locusFP=total_qloci-locusTP;
+		locusAFP=total_qloci-locusAQTP;
+		locusFP=total_qloci-locusQTP;
 		locusAFN=total_rloci-locusATP;
 		locusFN=total_rloci-locusTP;
 	}
@@ -717,7 +804,9 @@ public:
 		mrnaTP+=s.mrnaTP;
 		mrnaATP+=s.mrnaATP;
 		locusTP+=s.locusTP;
+    locusQTP+=s.locusQTP;
 		locusATP+=s.locusATP;
+    locusAQTP+=s.locusAQTP;
 		m_exons+=s.m_exons;
 		w_exons+=s.w_exons;
 		m_introns+=s.m_introns;
@@ -745,8 +834,9 @@ public:
 };
 
 class GSeqData {
-public:
 	int gseq_id;
+public:
+    const char* gseq_name;
     GList<GffObj> refs_f; //forward strand mRNAs
     GList<GffObj> refs_r; //reverse strand mRNAs
 	GList<GffObj> mrnas_f; //forward strand mRNAs
@@ -762,6 +852,9 @@ public:
 	GList<GLocus> nloci_u; //"novel" loci with no orientation found
 	
 	GList<CTData> tdata; //transcript data (uptr holder for all mrnas here)
+
+	int get_gseqid() { return gseq_id; }
+
 	//--<
 	GSeqData(int gid=-1):mrnas_f(true,true,false),mrnas_r(true,true,false),
 	loci_f(true,true,true),loci_r(true,true,true),
@@ -769,6 +862,8 @@ public:
 	nloci_f(true,false,true), nloci_r(true,false,true),
 	umrnas(true,true,false), nloci_u(true,true,true), tdata(false,true,false) {
 		gseq_id=gid;
+		if (gseq_id>=0) 
+		  gseq_name=GffObj::names->gseqs.getName(gseq_id);
 	}
 	bool operator==(GSeqData& d){
 		return (gseq_id==d.gseq_id);
@@ -958,19 +1053,21 @@ class GXConsensus:public GSeg {
    int id; //XConsensus ID
    int tss_id; //group id for those xconsensi with shared first exon
    int p_id; //group id for those xconsensi with "similar" protein
-   GffObj* tcons; //chosen (longest) transcript to define the "consensus"
+   GffObj* tcons; //longest transcript to represent the combined "consensus" structure
    GffObj* ref; //overlapping reference transcript 
    char refcode; // the code for ref relationship (like in the tracking file)
    char* aa;
    int aalen;
+   GXConsensus* contained; //if contained into another GXConsensus 
    //list of ichain-matching query (cufflinks) transcripts that contributed to this consensus
    GList<GffObj> qchain;
-   GXConsensus(GffObj* c, GList<GffObj> qlst, GffObj* r=NULL, char rcode=0)
+   GXConsensus(GffObj* c, CEqList* qlst, GffObj* r=NULL, char rcode=0)
                    :qchain(false,false,false) {
       ref=r;
       refcode=rcode;
       tcons=c;
-      qchain.Add(qlst);
+      if (qlst!=NULL) qchain.Add(*((GList<GffObj>*)qlst));
+                 else qchain.Add(c);
       count++;
       tss_id=0;
       p_id=0;
@@ -979,6 +1076,7 @@ class GXConsensus:public GSeg {
       aa=NULL;
       start=tcons->start;
       end=tcons->end;
+      contained=NULL;
       }
    ~GXConsensus() {
      if (aa!=NULL) GFREE(aa);
@@ -1057,7 +1155,7 @@ class GXLocus:public GSeg {
           j++; //check the next locus.mexon
         }//while mexons
       if (ovlexons.Count()==0) return false;
-      if (strand=='.' && loc->mrna_maxcov->strand>0)
+      if (strand=='.' && loc->mrna_maxcov->strand!='.')
              strand=loc->mrna_maxcov->strand;
       //have exon overlap:
       //-- add the rest of the non-overlapping mexons:
@@ -1131,12 +1229,64 @@ class GXLocus:public GSeg {
          rloci.Add(oxloc.rloci[i]);
          oxloc.rloci[i]->xlocus=this;
          }
+  } //::addMerge()
+
+
+ void checkContainment() {
+   //checking containment
+  for (int j=0;j<tcons.Count()-1;j++) {
+    GXConsensus* t=tcons[j];
+    for (int i=j+1;i<tcons.Count();i++) {
+       if (tcons[i]->contained!=NULL && t->tcons->exons.Count()>1) continue; //will check the container later anyway
+       int c_status=checkXConsContain(t->tcons, tcons[i]->tcons);
+       if (c_status==0) continue; //no containment relationship between t and tcons[i]
+       if (c_status>0) { //t is a container for tcons[i]
+            tcons[i]->contained=t;
+            }
+         else { //contained into exising XCons
+            t->contained=tcons[i];
+            break;
+            }
+       }
+   }
   }
-};
+ 
+ int checkXConsContain(GffObj* a, GffObj* b) {
+  // returns  1 if a is the container of b
+  //         -1 if a is contained in b
+  //          0 if no 
+  if (a->end<b->start || b->end<a->start) return 0;
+  if (a->exons.Count()==b->exons.Count()) {
+     if (a->exons.Count()>1) return 0; //same number of exons - no containment possible
+                                       //because equivalence was already tested
+           else { //single exon containment testing
+             //this is fuzzy and messy (end result may vary depending on the testing order)
+             int ovlen=a->exons[0]->overlapLen(b->exons[0]);
+             int minlen=GMIN(a->covlen, b->covlen);
+             if (ovlen>=minlen*0.8) { //if at least 80% of the shorter one is covered, it is contained
+                return ((a->covlen>b->covlen) ? 1 : -1);
+                }
+              else return 0;
+             //if (a->start<=b->start+10 && a->end+10>=b->end) return 1;
+             //  else { if (b->start<=a->start+10 && b->end+10>=a->end) return -1;
+             //          else return 0; 
+             //}
+             }
+     }
+   //different number of exons:
+   if (a->exons.Count()>b->exons.Count()) return t_contains(*a, *b) ? 1:0;
+                     else return t_contains(*b, *a) ? -1 : 0;
+  }
+  
+ void addXCons(GXConsensus* t) {
+  tcons.Add(t);
+  }
+  
+}; //GXLocus
 
 
 
-int parse_mRNAs(GList<GffObj>& mrnas,
+int parse_mRNAs(GfList& mrnas,
 				 GList<GSeqData>& glstdata,
 				 bool is_ref_set=true,
 				 bool check_for_dups=false,
@@ -1145,18 +1295,20 @@ int parse_mRNAs(GList<GffObj>& mrnas,
 //reading a mRNAs from a gff file and grouping them into loci
 void read_mRNAs(FILE* f, GList<GSeqData>& seqdata, GList<GSeqData>* ref_data=NULL, 
               bool check_for_dups=false, int qfidx=-1, const char* fname=NULL, 
-              bool checkseq=false, bool only_multiexon=false);
+              bool only_multiexon=false);
 
-void read_transcripts(FILE* f, GList<GSeqData>& seqdata);
+void read_transcripts(FILE* f, GList<GSeqData>& seqdata, bool keepAttrs=true);
+void sort_GSeqs_byName(GList<GSeqData>& seqdata);
 
-bool tMatch(GffObj& a, GffObj& b, int& ovlen, bool equnspl=false);
 
-//"position" a given coordinate x within a list of transcripts sorted by their start (lowest)
-//coordinate, using quick-search; the returned int is the list index of the closest *higher*
-//GffObj - i.e. starting right *ABOVE* the given coordinate
-//Convention: returns -1 if there is no such GffObj (i.e. last GffObj starts below x)
+bool tMatch(GffObj& a, GffObj& b, int& ovlen, bool fuzzunspl=false, bool contain_only=false);
+
+//use qsearch to "position" a given coordinate x within a list of transcripts sorted 
+//by their start (lowest) coordinate; the returned int is the list index of the 
+//closest GffObj starting just *ABOVE* coordinate x
+//Convention: returns -1 if there is no such GffObj (i.e. last GffObj start <= x)
 int qsearch_mrnas(uint x, GList<GffObj>& mrnas);
-int qsearch_segs(uint x, GList<GSeg>& segs); // same as above, but for GSeg lists
+int qsearch_loci(uint x, GList<GLocus>& segs); // same as above, but for GSeg lists
 
 GSeqData* getRefData(int gid, GList<GSeqData>& ref_data); //returns reference GSeqData for a specific genomic sequence
 
