@@ -19,6 +19,7 @@
 #include <bam/sam.h>
 
 #include "common.h"
+#include "multireads.h"
 
 using namespace std;
 using boost::shared_ptr;
@@ -57,20 +58,21 @@ struct CigarOp
 };
 
 typedef uint64_t InsertID;
-typedef uint32_t RefID;
+typedef uint64_t RefID;
 
 extern int num_deleted;
 
 /*  Stores the information from a single record of the bowtie map. A given read
-    may have many of these.  Reads up to 255bp are supported. 
+    may have many of these.
 */
 struct ReadHit
 {
 	ReadHit() : 
 		_ref_id(0),
 		_insert_id(0),
-		_error_prob(1.0),
-		_edit_dist(0xFFFFFFFF)
+        _base_mass(1.0),
+        _edit_dist(0xFFFFFFFF),
+		_num_hits(1)
     {
         num_deleted++;
     }
@@ -79,12 +81,13 @@ struct ReadHit
 			InsertID insert_id, 
 			int left, 
 			int read_len, 
-			bool antisense,
 			CuffStrand source_strand,
 			RefID partner_ref,
 			int partner_pos,
-			double error_prob,
-			unsigned int edit_dist) :
+			unsigned int edit_dist,
+			int num_hits,
+            float base_mass,
+            uint32_t sam_flag) :
 		_ref_id(ref_id),
 		_insert_id(insert_id), 
 		_left(left), 
@@ -92,9 +95,10 @@ struct ReadHit
 		_partner_pos(partner_pos),
 		_cigar(vector<CigarOp>(1,CigarOp(MATCH,read_len))),
 		_source_strand(source_strand),
-		_antisense_aln(antisense),
-		_error_prob(error_prob),
-		_edit_dist(edit_dist)
+        _base_mass(base_mass),
+        _edit_dist(edit_dist),
+		_num_hits(num_hits),
+        _sam_flag(sam_flag)
 	{
 		assert(_cigar.capacity() == _cigar.size());
 		_right = get_right();
@@ -105,12 +109,13 @@ struct ReadHit
 			InsertID insert_id, 
 			int left,  
 			const vector<CigarOp>& cigar,
-			bool antisense_aln,
 			CuffStrand source_strand,
 			RefID partner_ref,
 			int partner_pos, 
-			double error_prob,
-			unsigned int  edit_dist) : 
+			unsigned int  edit_dist,
+			int num_hits,
+            float base_mass,
+            uint32_t sam_flag) : 
 		_ref_id(ref_id),
 		_insert_id(insert_id), 	
 		_left(left),
@@ -118,9 +123,10 @@ struct ReadHit
 		_partner_pos(partner_pos),
 		_cigar(cigar),
 		_source_strand(source_strand),
-		_antisense_aln(antisense_aln),
-		_error_prob(error_prob),
-		_edit_dist(edit_dist)
+        _base_mass(base_mass),
+        _edit_dist(edit_dist),
+		_num_hits(num_hits),
+        _sam_flag(sam_flag)
 	{
 		assert(_cigar.capacity() == _cigar.size());
 		_right = get_right();
@@ -130,16 +136,17 @@ struct ReadHit
     ReadHit(const ReadHit& other)
     {
         _ref_id = other._ref_id;
-		_insert_id = other._insert_id; 	
+		_insert_id = other._insert_id;
 		_left = other._left;
 		_partner_ref_id = other._partner_ref_id;
 		_partner_pos = other._partner_pos;
 		_cigar = other._cigar;
 		_source_strand = other._source_strand;
-		_antisense_aln = other._antisense_aln;
-		_error_prob = other._error_prob;
-		_edit_dist = other._edit_dist;
+		_num_hits = other._num_hits;
+        _base_mass = other._base_mass;
+        _edit_dist = other._edit_dist;
         _right = get_right();
+        _sam_flag = other._sam_flag;
         num_deleted++;
     }
     
@@ -171,15 +178,26 @@ struct ReadHit
 	
 	bool contains_splice() const
 	{
-                // FIXME: This won't be true once we start supporting INDEL operators in CIGAR strings. See Ticket #183
-		return (_cigar.size() > 1);
+		for (size_t i = 0; i < _cigar.size(); ++i)
+		{
+				if (_cigar[i].opcode == REF_SKIP)
+					return true;
+		}
+		return false;
+	}
+	
+	bool is_singleton() const
+	{ 
+		return (partner_ref_id() == 0 ||
+				partner_ref_id() != ref_id() ||
+				abs(partner_pos() - left()) > max_partner_dist);
 	}
 	
 	bool operator==(const ReadHit& rhs) const
 	{
 	    return (_insert_id == rhs._insert_id &&
 	            _ref_id == rhs._ref_id &&
-	            _antisense_aln == rhs._antisense_aln &&
+	            antisense_align() == rhs.antisense_align() &&
 	            _left == rhs._left && 
 	            _source_strand == rhs._source_strand &&
 	            /* DO NOT USE ACCEPTED IN COMPARISON */
@@ -195,9 +213,19 @@ struct ReadHit
 	int left() const					{ return _left;				}
 	int right() const					{ return _right;			}
 	CuffStrand source_strand()	const	{ return _source_strand; }
-	bool antisense_align() const		{ return _antisense_aln;	}
-		
-	double error_prob() const			{ return _error_prob;		}
+	bool antisense_align() const		{ return _sam_flag & 0x10;	}
+    bool is_first() const               { return _sam_flag & 0x40;  }	
+	
+	// Number of multi-hits for this read
+	int num_hits() const				{ return _num_hits;			}
+	
+    // We are ignoring the _base_mass and re-calculating based on multi-hits
+	double mass() const 
+	{
+		if (is_singleton())
+			return 1.0/_num_hits;
+		return 0.5 / _num_hits;
+	}
 	
 	// For convenience, if you just want a copy of the gap intervals
 	// for this hit.
@@ -242,10 +270,12 @@ struct ReadHit
 		return _cigar.size() == 1 && _cigar[0].opcode == MATCH;
 	}
 	
-	unsigned int  edit_dist() const { return _edit_dist; }
-	
-	const string& hitfile_rec() const { return _hitfile_rec; }
-	void hitfile_rec(const string& rec) { _hitfile_rec = rec; }
+    unsigned int  edit_dist() const { return _edit_dist; }
+    
+    void trim(int trimmed_length);
+    
+	//const string& hitfile_rec() const { return _hitfile_rec; }
+	//void hitfile_rec(const string& rec) { _hitfile_rec = rec; }
 	
 private:
 	
@@ -285,10 +315,11 @@ private:
 	vector<CigarOp> _cigar;
 	
 	CuffStrand _source_strand;    // Which strand the read really came from, if known
-	bool _antisense_aln;       // Whether the alignment is to the reverse strand
-	double _error_prob;		   // Probability that this alignment is incorrect
-	unsigned int  _edit_dist;            // Number of mismatches
-	string _hitfile_rec; // Points to the buffer for the record from which this hit came
+    float _base_mass;
+    unsigned int  _edit_dist;            // Number of mismatches
+	int _num_hits; // Number of multi-hits (1 by default)
+    uint32_t _sam_flag;
+	//string _hitfile_rec; // Points to the buffer for the record from which this hit came
 };
 
 class ReadTable
@@ -304,7 +335,13 @@ public:
 		assert(_id);
 		return _id;
 	}
-	
+    
+    // Calculate checksum
+	InsertID get_cs(const string& name)
+	{
+        return string_checksum(name.c_str());
+    }
+    
 private:
 	
 	// This is FNV-1, see http://en.wikipedia.org/wiki/Fowler_Noll_Vo_hash
@@ -318,6 +355,17 @@ private:
 		}
 		return hash;
 	}
+    
+    
+    inline uint64_t string_checksum(const char * s)
+    {
+        uint64_t c = 0;
+        for ( ; *s; ++s)       
+        {
+            c += *s;
+        }
+        return c;
+    }
 };
 
 class RefSequenceTable
@@ -366,7 +414,7 @@ public:
 #if ENABLE_THREADS
         table_lock.lock();
 #endif
-		uint32_t _id = hash_string(name.c_str());
+		uint64_t _id = hash_string(name.c_str());
 		pair<InvertedIDTable::iterator, bool> ret = 
 		_by_id.insert(make_pair(_id, SequenceInfo(_next_id, NULL, NULL)));
 		if (ret.second == true)
@@ -475,12 +523,12 @@ public:
 private:
 	
 	// This is FNV-1, see http://en.wikipedia.org/wiki/Fowler_Noll_Vo_hash
-	inline uint32_t hash_string(const char* __s)
+	inline uint64_t hash_string(const char* __s)
 	{
-		uint32_t hash = 0x811c9dc5;
+		uint64_t hash = 0xcbf29ce484222325ull;
 		for ( ; *__s; ++__s)
 		{
-			hash *= 16777619;
+			hash *= 1099511628211ull;
 			hash ^= *__s;
 		}
 		return hash;
@@ -529,23 +577,25 @@ public:
 					   const string& ref_name,
 					   int left,
 					   const vector<CigarOp>& cigar,
-					   bool antisense_aln,
 					   CuffStrand source_strand,
 					   const string& partner_ref,
 					   int partner_pos,
-					   double error_prob,
-					   unsigned int  edit_dist);
+					   unsigned int  edit_dist,
+					   int num_hits,
+                       float base_mass,
+                       uint32_t sam_flag);
 	
 	ReadHit create_hit(const string& insert_name, 
 					   const string& ref_name,
 					   uint32_t left,
 					   uint32_t read_len,
-					   bool antisense_aln,
 					   CuffStrand source_strand,
 					   const string& partner_ref,
 					   int partner_pos,
-					   double error_prob,
-					   unsigned int  edit_dist);
+					   unsigned int  edit_dist,
+					   int num_hits,
+                       float base_mass,
+                       uint32_t sam_flag);
 	
 	virtual void reset() = 0;
 	
@@ -589,8 +639,7 @@ private:
     
 	ReadTable& _insert_table;
 	RefSequenceTable& _ref_table;
-    size_t _num_seq_header_recs;
-
+    uint32_t _num_seq_header_recs;
 };
 
 /******************************************************************************
@@ -775,11 +824,10 @@ public:
     _left_alignment(NULL),
     _right_alignment(NULL),
     _collapse_mass(0.0),
-    _is_mapped(false),
-    _collapsed_to(NULL){}
+    _is_mapped(false){}
     
 	MateHit(shared_ptr<ReadGroupProperties const> rg_props,
-            uint32_t refid, 
+            RefID refid, 
 			const ReadHit* left_alignment, 
 			const ReadHit* right_alignment) : 
     _rg_props(rg_props),
@@ -787,8 +835,7 @@ public:
 	_left_alignment(left_alignment),
 	_right_alignment(right_alignment),
 	_collapse_mass(0.0),
-	_is_mapped(false),
-	_collapsed_to(NULL)
+	_is_mapped(false)
 	{
 		//_expected_inner_dist = min(genomic_inner_dist(), _expected_inner_dist);
 	}
@@ -817,18 +864,23 @@ public:
 	void is_mapped(bool mapped) 
 	{ 
 		_is_mapped = mapped; 
-		if (_collapsed_to)
-			_collapsed_to->is_mapped(mapped);
 	}
 	
-	MateHit* collapsed_to() const { return _collapsed_to; }
-	void collapsed_to(MateHit* hit) { _collapsed_to = hit; }
+	int num_hits() const 
+	{
+		assert(_left_alignment);
+		return _left_alignment->num_hits();
+	}
 	
+	bool is_multi() const 
+	{
+		return num_hits() > 1;
+	}
+		
 	bool is_pair() const
 	{
 		return (_left_alignment && _right_alignment);
 	}
-	
 
 	int left() const 
 	{
@@ -877,6 +929,13 @@ public:
 	}
 	
 	
+	bool contains_splice() const
+	{
+		if (_right_alignment)
+			return (_left_alignment->contains_splice() || _right_alignment->contains_splice());
+		return (_left_alignment->contains_splice());
+	}
+	
 	InsertID insert_id() const
 	{
 		if (_left_alignment) return _left_alignment->insert_id();
@@ -921,13 +980,29 @@ public:
 		return make_pair(-1,-1);
 	}
 	
-	double error_prob() const
+	// MRT is incorrect and not added to rg_props until after inspect_map
+    // We are ignoring the mass reported by the ReadHits and re-calculating based on multi-hits
+	double mass() const
 	{
-		if (_left_alignment)
-			return _left_alignment->error_prob();
-		else if (_right_alignment)
-			return _right_alignment->error_prob();
-		return 1.0;
+        double base_mass = 1.0;
+
+        if (is_multi())
+		{
+			shared_ptr<MultiReadTable> mrt = _rg_props->multi_read_table();
+			if (mrt)
+				return mrt->get_mass(*this);
+			else
+				return base_mass/num_hits();
+		}
+		return base_mass;
+	}
+    
+	double common_scale_mass() const
+	{
+       	double m = mass();
+        m *= _rg_props->mass_scale_factor();
+        
+        return m;
 	}
 	
 	unsigned int  edit_dist() const
@@ -940,7 +1015,6 @@ public:
 		return edits;
 	}
 	
-	double mass() const { return (1.0 - error_prob()); }
 	double collapse_mass() const { return _collapse_mass; }
 	void collapse_mass(double m) { _collapse_mass = m; }
 	void incr_collapse_mass(double incr) { _collapse_mass += incr; }
@@ -953,7 +1027,6 @@ private:
 	const ReadHit* _right_alignment;
 	double _collapse_mass;
 	bool _is_mapped;
-	MateHit* _collapsed_to;
 	//bool _closed;
 };
 
@@ -961,6 +1034,9 @@ bool mate_hit_lt(const MateHit& lhs, const MateHit& rhs);
 
 bool hits_eq_mod_id(const ReadHit& lhs, const ReadHit& rhs);
 
+bool hits_eq_non_multi(const MateHit& lhs, const MateHit& rhs);
+bool hits_eq_non_multi_non_replicate(const MateHit& lhs, const MateHit& rhs);
+
 bool hits_equals(const MateHit& lhs, const MateHit& rhs);
 
 bool has_no_collapse_mass(const MateHit& hit);
