@@ -19,11 +19,15 @@ namespace ublas = boost::numeric::ublas;
 
 double entropy(const ublas::vector<double>& p);
 
-double jensen_shannon_div(std::vector<ublas::vector<double> >& sample_kappas);
+double jensen_shannon_distance(std::vector<ublas::vector<double> >& sample_kappas);
+
+//void alt_jensen_shannon_gradient(std::vector<ublas::vector<double> >& sample_kappas,
+//                                 double js,
+//                                 ublas::vector<double>& gradient);
 
 void jensen_shannon_gradient(std::vector<ublas::vector<double> >& sample_kappas,
 							 double js,
 							 ublas::vector<double>& gradient);
 
 void make_js_covariance_matrix(std::vector<ublas::matrix<double> >& kappa_covariances,
-							   ublas::matrix<double>& js_covariance);
\ No newline at end of file
+							   ublas::matrix<double>& js_covariance);
