@@ -9,15 +9,16 @@
 
 #include <list>
 #include <algorithm>
+#include <numeric>
 #include "common.h"
 #include "scaffolds.h"
 
 using namespace std;
 
 bool AugmentedCuffOp::compatible(const AugmentedCuffOp& lhs,
-								 const AugmentedCuffOp& rhs)
+								 const AugmentedCuffOp& rhs,
+								 int overhang_tolerance)
 {
-	int l_match = match_length(lhs, rhs.g_left(), rhs.g_right());
 	if (rhs.opcode == CUFF_INTRON)
 	{
 		if (lhs.opcode == CUFF_INTRON)
@@ -28,23 +29,27 @@ bool AugmentedCuffOp::compatible(const AugmentedCuffOp& lhs,
 		}
 		else if (lhs.opcode == CUFF_UNKNOWN)
 		{
-            int left_diff = abs(lhs.g_left() - rhs.g_left());
-            int right_diff = abs(lhs.g_right() - rhs.g_right());
-				if (left_diff + right_diff > max_frag_len)
-					return false;
-			}
-		else if (l_match > bowtie_overhang_tolerance)
-		{
-			return false;
-		}
+            //int left_diff = abs(lhs.g_left() - rhs.g_left());
+            //int right_diff = abs(lhs.g_right() - rhs.g_right());
+			//if (left_diff + right_diff > max_frag_len)
+            //    return false;
+        }
+        else
+        {
+            int l_match = match_length(lhs, rhs.g_left(), rhs.g_right());
+            if (l_match > overhang_tolerance)
+            {
+                return false;
+            }
+        }
 	}
 	else if (rhs.opcode == CUFF_UNKNOWN)
 	{
+        int l_match = match_length(lhs, rhs.g_left(), rhs.g_right());
 		if (l_match > max_frag_len)
 			return false;
 	}
-	
-	int r_match = match_length(rhs, lhs.g_left(), lhs.g_right());
+
 	if (lhs.opcode == CUFF_INTRON)
 	{
 		if (rhs.opcode == CUFF_INTRON)
@@ -55,18 +60,23 @@ bool AugmentedCuffOp::compatible(const AugmentedCuffOp& lhs,
 		}
 		else if (lhs.opcode == CUFF_UNKNOWN)
 		{
-            int left_diff = abs(lhs.g_left() - rhs.g_left());
-            int right_diff = abs(lhs.g_right() - rhs.g_right());
-				if (left_diff + right_diff > max_frag_len)
+            //int left_diff = abs(lhs.g_left() - rhs.g_left());
+            //int right_diff = abs(lhs.g_right() - rhs.g_right());
+            //if (left_diff + right_diff > max_frag_len)
+            //    return false;
+		}
+		else 
+        {
+            int r_match = match_length(rhs, lhs.g_left(), lhs.g_right());
+            if (r_match > overhang_tolerance)
+            {
                 return false;
-		}
-		else if (r_match > bowtie_overhang_tolerance)
-		{
-			return false;
-		}
+            }
+        }
 	}	
 	else if (lhs.opcode == CUFF_UNKNOWN)
 	{
+        int r_match = match_length(rhs, lhs.g_left(), lhs.g_right());
 		if (r_match > max_frag_len)
 			return false;
 	}
@@ -127,7 +137,8 @@ void record_gaps(const vector<AugmentedCuffOp>& to_fill,
 // this function
 void AugmentedCuffOp::fill_interstices(vector<AugmentedCuffOp>& to_fill,
                                        const vector<AugmentedCuffOp>& filler,
-                                       bool allow_flank_fill)
+                                       bool allow_flank_fill,
+                                       bool allow_flanking_introns)
 {
 	vector<AugmentedCuffOp> filled = to_fill;
 	vector<pair<int, int> > gaps;
@@ -271,20 +282,21 @@ void AugmentedCuffOp::fill_interstices(vector<AugmentedCuffOp>& to_fill,
 	}
     
 	sort(filled.begin(), filled.end(), g_left_lt);
-    
-//    for (size_t i = 0; i < filled.size(); ++i)
-//    {
-//        if (filled[i].opcode == CUFF_INTRON)
-//        {
-//            assert (i > 0);
-//            assert (i < filled.size() -1);
-//            assert (filled[i-1].opcode == CUFF_MATCH);
-//            assert (filled[i+1].opcode == CUFF_MATCH);
-//            assert (filled[i-1].g_right() == filled[i].g_left());
-//            assert (filled[i+1].g_left() == filled[i].g_right());
-//        }
-//    }
-    
+    if (!allow_flanking_introns)
+    {
+        for (size_t i = 0; i < filled.size(); ++i)
+        {
+            if (filled[i].opcode == CUFF_INTRON)
+            {
+                assert (i > 0);
+                assert (i < filled.size() -1);
+                assert (filled[i-1].opcode == CUFF_MATCH);
+                assert (filled[i+1].opcode == CUFF_MATCH);
+                assert (filled[i-1].g_right() == filled[i].g_left());
+                assert (filled[i+1].g_left() == filled[i].g_right());
+            }
+        }
+    }
 	to_fill = filled;
 }
 
@@ -293,7 +305,8 @@ void AugmentedCuffOp::fill_interstices(vector<AugmentedCuffOp>& to_fill,
 // ops is assumed to be sorted
 void AugmentedCuffOp::merge_ops(const vector<AugmentedCuffOp>& ops, 
                                 vector<AugmentedCuffOp>& merged,
-                                bool introns_overwrite_matches)
+                                bool introns_overwrite_matches,
+                                bool allow_flank_introns)
 {	
 #if DEBUG
 	//assert(std::adjacent_find(ops.begin(), ops.end(), g_left_lt) == ops.end());
@@ -356,12 +369,13 @@ void AugmentedCuffOp::merge_ops(const vector<AugmentedCuffOp>& ops,
     }
     if (gaurd_assembly() && merged_length > (int)max_gene_length)
 	{
-		fprintf(stderr, "Warning: nonsense gene merge - product > max_gene_length\n");
+		fprintf(stderr, "Error: nonsense gene merge - product > max_gene_length\n");
 		exit(1);
 	}
 	
 	unknowns.push_back(AugmentedCuffOp(CUFF_UNKNOWN, g_min, g_max - g_min + 1));
 	
+    vector<AugmentedCuffOp> pre_merge = matches;
 	
 	disjoint_ops(matches);
 	disjoint_ops(introns);
@@ -371,37 +385,39 @@ void AugmentedCuffOp::merge_ops(const vector<AugmentedCuffOp>& ops,
 	if (introns_overwrite_matches)
 	{
 		merged = introns;
-		fill_interstices(merged, matches, true);
+		fill_interstices(merged, matches, true, allow_flank_introns);
 		vector<pair<int, int> > gaps;
 		record_gaps(merged, gaps);
 		if (!gaps.empty())
-			fill_interstices(merged, unknowns, false); 
+			fill_interstices(merged, unknowns, false, allow_flank_introns); 
 	}
 	else
 	{
 		merged = matches;
-		fill_interstices(merged, introns, false);
+		fill_interstices(merged, introns, false, allow_flank_introns);
 		vector<pair<int, int> > gaps;
 		record_gaps(merged, gaps);
 		if (!gaps.empty())
-			fill_interstices(merged, unknowns, false); 
+			fill_interstices(merged, unknowns, false, allow_flank_introns); 
 	}
 
-    //FIXME: put these back
-//    assert (merged.front().opcode == CUFF_MATCH);
-//    assert (merged.back().opcode == CUFF_MATCH);
-//    
-//    for (size_t i = 1; i < merged.size(); ++i)
-//    {
-//        if (merged[i].opcode == CUFF_INTRON)
-//        {
-//            assert (merged[i-1].opcode == CUFF_MATCH);
-//        }
-//        else if (merged[i].opcode == CUFF_UNKNOWN)
-//        {
-//            assert (merged[i-1].opcode != CUFF_INTRON);
-//        }
-//    }
+    if (!allow_flank_introns)
+    {
+        assert (merged.front().opcode == CUFF_MATCH);
+        assert (merged.back().opcode == CUFF_MATCH);
+    }
+    for (size_t i = 1; i < merged.size(); ++i)
+    {
+		assert(merged[i-1].g_right() == merged[i].g_left());
+        if (merged[i].opcode == CUFF_INTRON)
+        {
+            assert (merged[i-1].opcode == CUFF_MATCH);
+        }
+        else if (merged[i].opcode == CUFF_UNKNOWN)
+        {
+            assert (merged[i-1].opcode != CUFF_INTRON);
+        }
+    }
 }
     
 
@@ -437,7 +453,7 @@ bool check_merge_length(const vector<AugmentedCuffOp>& ops)
 		}
 
 inline bool has_intron(const Scaffold& scaff)
-			{
+{
 			
 	const vector<AugmentedCuffOp>& ops = scaff.augmented_ops();
 	for (size_t j = 0; j < ops.size(); ++j)
@@ -462,6 +478,219 @@ inline bool has_intron(const Scaffold& scaff)
 //	return false;
 //}
 
+void Scaffold::extend_5(const Scaffold& other)
+{
+	assert(Scaffold::compatible(*this, other, ref_merge_overhang_tolerance));
+	
+	if (strand() == CUFF_FWD)
+	{
+		AugmentedCuffOp& my_first_op = _augmented_ops.front();
+		const AugmentedCuffOp& other_first_op = other.augmented_ops().front();
+
+		my_first_op.g_left(other_first_op.g_left());
+	}
+	else if (strand() == CUFF_REV)
+	{
+		AugmentedCuffOp& my_last_op = _augmented_ops.back();
+		const AugmentedCuffOp& other_last_op = other.augmented_ops().back();
+		
+		my_last_op.g_right(other_last_op.g_right());
+	}
+	else 
+	{
+		assert(false);
+	}
+	
+	int id_length = annotated_trans_id().length();
+	if (id_length < 4 || _annotated_trans_id.substr(id_length-4)!="_ext")
+		_annotated_trans_id += "_ext";
+}
+
+// Clip final 3' exon by given amount
+void Scaffold::trim_3(int to_remove)
+{
+	if(strand() == CUFF_FWD)
+	{
+		AugmentedCuffOp& exon_3 = _augmented_ops.back();
+		assert (exon_3.genomic_length > to_remove);
+		exon_3.genomic_length -= to_remove;
+	}
+	else if(strand() == CUFF_REV)
+	{
+		AugmentedCuffOp& exon_3 = _augmented_ops.front();
+		assert (exon_3.genomic_length > to_remove);
+		exon_3.genomic_offset += to_remove;
+		exon_3.genomic_length -= to_remove;
+
+	}
+	else
+	{
+		assert(false);
+	}
+}
+
+// Clip final 3' exon by given amount
+void Scaffold::extend_3(int to_add)
+{
+	if(strand() == CUFF_FWD)
+	{
+		AugmentedCuffOp& exon_3 = _augmented_ops.back();
+		//assert (exon_3.genomic_length > to_remove);
+		exon_3.genomic_length += to_add;
+	}
+	else if(strand() == CUFF_REV)
+	{
+		AugmentedCuffOp& exon_3 = _augmented_ops.front();
+		//assert (exon_3.genomic_length > to_remove);
+		exon_3.genomic_offset -= to_add;
+		exon_3.genomic_length += to_add;
+        
+	}
+	else
+	{
+		assert(false);
+	}
+}
+
+void Scaffold::tile_with_scaffs(vector<Scaffold>& tile_scaffs, int max_len, int tile_offset) const
+{
+    int min_len = tile_offset;
+	assert(min_len % tile_offset == 0 && max_len % tile_offset == 0);
+	
+	if (length() < max_len)
+		return;
+	
+	size_t l = 0;
+	size_t r = 0;
+	int l_off = 0;
+	int r_off = min_len; // One past end
+	int curr_len = min_len;
+	int remaining_len = length();
+	
+    const vector<AugmentedCuffOp>& orig_ops = augmented_ops();
+    
+	while(true)
+	{
+		while (l < orig_ops.size() && (orig_ops[l].opcode != CUFF_MATCH || l_off >= orig_ops[l].genomic_length))
+		{
+			if (orig_ops[l].opcode == CUFF_MATCH)
+				l_off -= orig_ops[l].genomic_length;
+            ++l;
+			//if(++l == augmented_ops().size())
+			//	assert(false);
+		}
+		while (r < orig_ops.size() && (orig_ops[r].opcode != CUFF_MATCH || r_off > orig_ops[r].genomic_length)) // Strictly > because r_off is one past
+		{
+			if (orig_ops[r].opcode == CUFF_MATCH)
+				r_off -= orig_ops[r].genomic_length;
+            ++r;
+			//if(++r == augmented_ops().size())
+			//	assert(false);
+		}
+		
+		vector<AugmentedCuffOp> ops;
+		
+        //if (l >= orig_ops.size() && r >= orig_ops.size())
+        //    break;
+        
+		if (l==r)
+		{
+            assert (l < orig_ops.size());
+			ops.push_back(AugmentedCuffOp(CUFF_MATCH, orig_ops[l].g_left() + l_off, r_off - l_off));
+		}
+		else
+		{
+            assert(orig_ops.size());
+            //assert(orig_ops[l].genomic_offset != 0);
+            
+			ops.push_back(AugmentedCuffOp(CUFF_MATCH, orig_ops[l].g_left() + l_off, orig_ops[l].genomic_length - l_off));
+			assert(ops.back().g_right() > ops.back().g_left());
+			for(size_t i = l+1; i < r; i++)
+			{
+				ops.push_back(orig_ops[i]);
+			}
+			if (r_off > 0 && r < orig_ops.size())
+            {
+                assert (r < orig_ops.size());
+                //assert (orig_ops[r].genomic_offset != 0);
+				ops.push_back(AugmentedCuffOp(CUFF_MATCH, orig_ops[r].g_left(), r_off));
+            }
+		}
+		assert(ops.back().g_right() > ops.back().g_left());
+
+        // genomic_offset actually could be zero - from an exon starting at coord
+        // 1 in some chromosome of the ref.
+//		foreach(const AugmentedCuffOp& op, ops)
+//        {
+//            assert (op.genomic_offset != 0);
+//        }
+        
+		tile_scaffs.push_back(Scaffold(this->ref_id(), this->strand(), ops, true)); 
+		assert(tile_scaffs.back().length() == curr_len );
+		assert(tile_scaffs.back().left() >= left() &&
+			   tile_scaffs.back().right() <= right());
+		
+		if (l==0 && l_off == 0 && curr_len < max_len) // On left end, not yet full length
+		{
+			curr_len += tile_offset;
+			r_off += tile_offset;
+		}
+		else if(remaining_len - tile_offset < max_len) // On the right end of transcript, decreasing in length
+		{
+			curr_len += augmented_ops().back().genomic_length - r_off - tile_offset; 
+			r_off = augmented_ops().back().genomic_length;
+			l_off += tile_offset;
+			remaining_len -= tile_offset;
+			if (curr_len < min_len)
+				return;
+		}
+		else // In the middle of the transcript, full length
+		{
+			l_off += tile_offset;
+			r_off += tile_offset;
+			remaining_len -= tile_offset;
+		}
+			
+	}
+	
+}
+
+bool Scaffold::sub_scaffold(Scaffold& sub_scaff, int g_left, int match_length) const	
+{
+	size_t i;
+	for(i = 0; i < augmented_ops().size(); ++i)
+	{
+		if (augmented_ops()[i].g_right() > g_left)
+			break;
+	}
+	const AugmentedCuffOp& l_op = augmented_ops()[i++];
+	
+	assert(l_op.opcode == CUFF_MATCH);
+	
+	vector<AugmentedCuffOp> sub_ops;
+	sub_ops.push_back(AugmentedCuffOp(CUFF_MATCH, g_left, min(match_length, l_op.g_right()-g_left)));					 
+	int len_so_far = sub_ops.back().genomic_length;
+						 
+	while(len_so_far < match_length && i < augmented_ops().size())
+	{
+		const AugmentedCuffOp& op = augmented_ops()[i++];
+		if(op.opcode==CUFF_MATCH)
+		{
+			sub_ops.push_back(AugmentedCuffOp(CUFF_MATCH, op.g_left(), min(match_length-len_so_far, op.genomic_length)));
+			len_so_far += sub_ops.back().genomic_length;
+			if (len_so_far >= match_length)
+				break;
+		}
+		else 
+		{
+			sub_ops.push_back(op);
+		}
+	}
+	
+	//assert(len_so_far == match_length);
+	sub_scaff = Scaffold(this->ref_id(), this->strand(), sub_ops, true);
+	return (len_so_far == match_length);
+}
 
 void Scaffold::merge(const Scaffold& lhs, 
 					 const Scaffold& rhs, 
@@ -496,14 +725,12 @@ void Scaffold::merge(const Scaffold& lhs,
 													  merged._mates_in_scaff.end());
 	merged._mates_in_scaff.erase(new_end, merged._mates_in_scaff.end());
 	
-	merged._right = merged._augmented_ops.back().g_right();
-	merged._left = merged._augmented_ops.front().g_left();
 	
-	int r_check = merged._left;
+	int r_check = merged.left();
 	for (size_t i = 0; i < merged._augmented_ops.size(); ++i)
 		r_check += merged._augmented_ops[i].genomic_length;
 	
-	assert(r_check == merged._right);
+	assert(r_check == merged.right());
 	
 	if (lhs.strand() != CUFF_STRAND_UNKNOWN)
 		merged._strand = lhs.strand();
@@ -511,7 +738,15 @@ void Scaffold::merge(const Scaffold& lhs,
 		merged._strand = rhs.strand();
 	
 	merged._has_intron = has_intron(merged);
-	assert(!merged.has_intron() || merged._strand != CUFF_STRAND_UNKNOWN);
+	assert(merged._strand != CUFF_STRAND_UNKNOWN || !merged.has_intron() );
+	assert(!merged.is_ref());
+    
+    if (library_type == "transfrags")
+    {
+        double avg_fpkm = lhs.fpkm() + rhs.fpkm();
+        avg_fpkm /= 2;
+        merged.fpkm(avg_fpkm);
+    }
 }
 
 
@@ -527,7 +762,6 @@ void Scaffold::merge(const vector<Scaffold>& s,
 	for (size_t i = 0; i < s.size(); ++i)
 	{
 		ops.insert(ops.end(), s[i]._augmented_ops.begin(), s[i]._augmented_ops.end());
-		
 		merged._mates_in_scaff.insert(merged._mates_in_scaff.end(),
 									  s[i]._mates_in_scaff.begin(), 
 									  s[i]._mates_in_scaff.end());
@@ -552,6 +786,7 @@ void Scaffold::merge(const vector<Scaffold>& s,
 	
 	AugmentedCuffOp::merge_ops(ops, merged._augmented_ops, introns_overwrite_matches);
 	
+	
 	//assert (ops.empty() || !(merged._augmented_ops.empty()));
 #ifdef DEBUG
 	if (merged._augmented_ops.empty() || 
@@ -564,25 +799,33 @@ void Scaffold::merge(const vector<Scaffold>& s,
 	
 	merged._ref_id = s.front().ref_id();
 	merged._strand = strand;
-	
-	merged._left = merged._augmented_ops.front().g_left();
-	merged._right = merged._augmented_ops.back().g_right();
-	
-	int r_check = merged._left;
+		
+	int r_check = merged.left();
 	for (size_t i = 0; i < merged._augmented_ops.size(); ++i)
 		r_check += merged._augmented_ops[i].genomic_length;
 	
 #ifdef DEBUG
-	if (r_check != merged._right)
+	if (r_check != merged.right())
 	{
 		AugmentedCuffOp::merge_ops(ops, merged._augmented_ops, introns_overwrite_matches);
 	}
 #endif
 	
-	assert (r_check == merged._right);
+	assert (r_check == merged.right());
 	merged._has_intron = has_intron(merged);
-	
-	assert(!merged.has_intron()|| merged._strand != CUFF_STRAND_UNKNOWN);
+
+	assert(merged._strand != CUFF_STRAND_UNKNOWN || !merged.has_strand_support());
+    
+    if (library_type == "transfrags")
+    {
+        double avg_fpkm = 0.0;
+        foreach (const Scaffold& sc, s)
+        {
+            avg_fpkm += sc.fpkm();
+        }
+        avg_fpkm /= s.size();
+        merged.fpkm(avg_fpkm);
+    }
 }
 
 void Scaffold::fill_gaps(int filled_gap_size)
@@ -609,6 +852,8 @@ void Scaffold::fill_gaps(int filled_gap_size)
 	
 	AugmentedCuffOp::merge_ops(ops, _augmented_ops, false);
 	_has_intron = has_intron(*this);
+	assert(!has_strand_support() || _strand != CUFF_STRAND_UNKNOWN);
+
 }
 
 void Scaffold::fill_gaps(const vector<AugmentedCuffOp>& filler)
@@ -617,24 +862,77 @@ void Scaffold::fill_gaps(const vector<AugmentedCuffOp>& filler)
 	
 	const vector<AugmentedCuffOp>& orig_ops = augmented_ops();
     
-	for (size_t i = 0; i < orig_ops.size(); ++i)
-	{
-		if (orig_ops[i].opcode == CUFF_UNKNOWN)
-		{
-            
-		}
-		else
-		{
-			ops.push_back(orig_ops[i]);
-		}
-	}
+	const vector<AugmentedCuffOp> og_ops = augmented_ops();
+	
+    vector<AugmentedCuffOp> unknowns;
+	
+    size_t g_max = 0;
+	size_t g_min = 0xFFFFFFFF;
     
-    AugmentedCuffOp::fill_interstices(ops, filler, false); 
+    vector<AugmentedCuffOp> tmp_filler = filler;
+    
+    foreach(const AugmentedCuffOp& op, orig_ops)
+    {
+		assert (op.g_left() < op.g_right());
+		
+		if ((size_t)op.g_left() < g_min)
+			g_min = op.g_left();
+		if ((size_t)op.g_right() > g_max)
+			g_max = op.g_right();
+	}
+
+	tmp_filler.push_back(AugmentedCuffOp(CUFF_UNKNOWN, g_min, g_max - g_min + 1));
+    sort(tmp_filler.begin(), tmp_filler.end(), AugmentedCuffOp::g_left_lt);
+    
+    vector<AugmentedCuffOp> padded_filler;
+    AugmentedCuffOp::merge_ops(tmp_filler, padded_filler, false);
+    	
+    vector<AugmentedCuffOp> overlapping;
+    foreach (const AugmentedCuffOp& op, padded_filler)
+    {
+		//if (left() <= op.g_left() && right() >= op.g_right()
+		if(::overlap_in_genome(op.g_left(),op.g_right(), left(), right())
+            && (op.opcode != CUFF_UNKNOWN || !overlapping.empty()))
+        {
+            overlapping.push_back(op);
+        }
+    }
+    
+    overlapping.insert(overlapping.end(), _augmented_ops.begin(),_augmented_ops.end()); 
+    sort(overlapping.begin(), overlapping.end(), AugmentedCuffOp::g_left_lt);
 	
-	sort(ops.begin(), ops.end(), AugmentedCuffOp::g_left_lt);
+	// Trim first in last in case they hang over the scaffold boundaries
+	if (overlapping.front().g_left() < left())
+	{
+		AugmentedCuffOp& first_op = overlapping.front();
+		first_op.genomic_length -= (left() - first_op.g_left());
+		first_op.genomic_offset = left();
+	}
+	if (overlapping.back().g_right() > right())
+	{
+		AugmentedCuffOp& last_op = overlapping.back();
+		last_op.genomic_length -= (last_op.g_right() - right());
+	}
+	if (overlapping.back().opcode == CUFF_INTRON && overlapping.back().genomic_length <= bowtie_overhang_tolerance)
+	{
+		overlapping.pop_back();
+	}
+	if (overlapping.front().opcode == CUFF_INTRON && overlapping.front().genomic_length <= bowtie_overhang_tolerance)
+	{
+		overlapping.erase(overlapping.begin(), overlapping.begin()+1);
+	}
 	
-	AugmentedCuffOp::merge_ops(ops, _augmented_ops, false);
+    // we don't want either the terminal ops in the filler to be unknowns,
+    // because they'll just propogate to the scaffold we're trying to fill.
+    if (!overlapping.empty() && overlapping.back().opcode == CUFF_UNKNOWN)
+        overlapping.pop_back();
+    
+    if (overlapping.empty())
+        return;
+    
+	AugmentedCuffOp::merge_ops(overlapping, _augmented_ops, true);
 	_has_intron = has_intron(*this);
+	assert(!has_strand_support() || _strand != CUFF_STRAND_UNKNOWN);
 }
 
 bool Scaffold::overlap_in_genome(const Scaffold& lhs, 
@@ -672,9 +970,32 @@ bool Scaffold::strand_agree(const Scaffold& lhs,
 	return strand;
 }
 
-	
+bool Scaffold::exons_overlap(const Scaffold& lhs, 
+							 const Scaffold& rhs)
+{
+	const vector<AugmentedCuffOp>& lhs_ops = lhs.augmented_ops();
+	const vector<AugmentedCuffOp>& rhs_ops = rhs.augmented_ops();
+
+	for (size_t l = 0; l < lhs_ops.size(); l++)
+	{
+		if (lhs_ops[l].opcode != CUFF_MATCH)
+			continue;
+		
+		for (size_t r = 0; r < rhs_ops.size(); r++)
+		{
+			if (rhs_ops[r].opcode == CUFF_MATCH &&
+				AugmentedCuffOp::overlap_in_genome(lhs_ops[l], rhs_ops[r]))
+			{
+				return true;
+			}
+		}
+	}
+	return false;
+}
+
 bool Scaffold::compatible(const Scaffold& lhs, 
-						  const Scaffold& rhs)
+						  const Scaffold& rhs,
+						  int overhang_tolerance)
 {	
 	if (!strand_agree(lhs, rhs))
 		return false;
@@ -684,7 +1005,7 @@ bool Scaffold::compatible(const Scaffold& lhs,
 		if (overlap_in_genome(lhs, rhs, olap_radius))
 		{
 			// check compatibility
-			if (!compatible_contigs(lhs, rhs))
+			if (!compatible_contigs(lhs, rhs, overhang_tolerance))
 				return false;
 		}
 	}
@@ -693,7 +1014,7 @@ bool Scaffold::compatible(const Scaffold& lhs,
 		if (overlap_in_genome(rhs, lhs, olap_radius))
 		{
 			// check compatibility
-			if (!compatible_contigs(rhs, lhs))
+			if (!compatible_contigs(rhs, lhs, overhang_tolerance))
 				return false;
 		}
 	}
@@ -702,8 +1023,9 @@ bool Scaffold::compatible(const Scaffold& lhs,
 	return true;
 }
 
-bool Scaffold::distance_compatible_contigs(const Scaffold& lhs, 
-										   const Scaffold& rhs)
+bool Scaffold::compatible_contigs(const Scaffold& lhs, 
+								  const Scaffold& rhs,
+								  int overhang_tolerance)
 {
 	const vector<AugmentedCuffOp>& l_aug = lhs._augmented_ops;
 	const vector<AugmentedCuffOp>& r_aug = rhs._augmented_ops;
@@ -722,7 +1044,7 @@ bool Scaffold::distance_compatible_contigs(const Scaffold& lhs,
 			if (AugmentedCuffOp::overlap_in_genome(l_op, r_op))
 			{
 				// check compatibility
-				if (!AugmentedCuffOp::compatible(l_op, r_op))
+				if (!AugmentedCuffOp::compatible(l_op, r_op, overhang_tolerance))
 					return false;
 //				if (l_op.opcode == CUFF_UNKNOWN && 
 //					r_op.opcode == CUFF_MATCH)
@@ -747,7 +1069,7 @@ bool Scaffold::distance_compatible_contigs(const Scaffold& lhs,
 			if (AugmentedCuffOp::overlap_in_genome(r_op, l_op))
 			{
 				// check compatibility
-				if (!AugmentedCuffOp::compatible(r_op, l_op))
+				if (!AugmentedCuffOp::compatible(r_op, l_op, overhang_tolerance))
 						return false;
 //				if (r_op.opcode == CUFF_UNKNOWN && 
 //					l_op.opcode == CUFF_MATCH)
@@ -1112,19 +1434,38 @@ pair <int,int> Scaffold::genomic_to_transcript_span(pair<int,int> g_span) const
 	return make_pair(s_start, s_end);
 }
 
-// Returns true only when both the start and end are found
+// Returns true only when both the start and end are found (ie, frag_len is known), which
+// can only happen if the read is paired.  Returned values that are equal to the trans_len 
+// should be ignored, as they are invalid.
 // We can't use EmpDist unless it is unpaired since we call this function in inspect_bundle
+// End is inclusive
+// Returned indices are oriented with the transript!
 bool Scaffold::map_frag(const MateHit& hit, int& start, int& end, int& frag_len) const
 {
 	
 	int trans_len = length();
-	
-	// Defaults will cause them to be ignored when they are unknown
+    
+//    if (Scaffold(hit).augmented_ops() == augmented_ops())
+//    {
+//        int a = 4;
+//    }
+    
 	start = trans_len;
 	end = trans_len;
 	
-	
-	if (hit.is_pair())
+    Scaffold h(hit);
+    if(!(contains(h) && Scaffold::compatible(*this, h)))
+       return false;
+    
+    if (hit.read_group_props()->complete_fragments())
+    {
+        pair<int,int> g_span = make_pair(hit.left(), hit.right() - 1);
+		pair<int,int> t_span = genomic_to_transcript_span(g_span);
+		start = t_span.first;
+		end = t_span.second;
+		frag_len = abs(end-start)+1;
+    }
+	else if (hit.is_pair())
 	{
 		pair<int,int> g_span = hit.genomic_outer_span();
 		pair<int,int> t_span = genomic_to_transcript_span(g_span);
@@ -1141,8 +1482,8 @@ bool Scaffold::map_frag(const MateHit& hit, int& start, int& end, int& frag_len)
 	{
 		shared_ptr<const EmpDist> frag_len_dist = hit.read_group_props()->frag_len_dist();
 
-		if (hit.left_alignment()->antisense_align() && strand() != CUFF_REV 
-			|| !(hit.left_alignment()->antisense_align()) && strand() == CUFF_REV)
+		if ((hit.left_alignment()->antisense_align() && strand() != CUFF_REV) 
+			|| (!hit.left_alignment()->antisense_align() && strand() == CUFF_REV))
 		{
 			int g_end  = (strand()!=CUFF_REV) ? hit.right()-1:hit.left();
 			end = genomic_to_transcript_coord(g_end);
@@ -1205,7 +1546,7 @@ void Scaffold::clear_hits()
 bool Scaffold::add_hit(const MateHit* hit)
 {
 	Scaffold hs(*hit);
-	if (Scaffold::overlap_in_genome(*this, hs, olap_radius) &&
+	if (contains(hs) &&
 		Scaffold::compatible(*this, hs))
 	{
 		if (!binary_search(_mates_in_scaff.begin(),
@@ -1250,12 +1591,12 @@ void Scaffold::get_complete_subscaffolds(vector<Scaffold>& complete)
 				int right_known;
 				
 				if (i == _augmented_ops.size() - 1)
-					right_known = _right;
+					right_known = right();
 				else
 					right_known = op.g_left() - 1;
 				
 				if (last_unknown == -1)
-					left_known = _left;
+					left_known = left();
 				else
 					left_known = _augmented_ops[last_unknown].g_right();
 				
@@ -1275,12 +1616,41 @@ void Scaffold::get_complete_subscaffolds(vector<Scaffold>& complete)
                     {
 
                         const MateHit& m = **mitr;
-                        if (left_known <= m.left() && m.right() < right_known)
+                        if (left_known <= m.left() && m.right() <= right_known)
                         {
                             known.add_hit(&m);
                         }
                     }
                     
+                   // if (known.has_intron())
+                        known.strand(_strand);
+                    
+                    //assert (!known.mate_hits().empty());
+                    //assert(!known.has_intron()|| known.strand() != CUFF_STRAND_UNKNOWN);
+                    
+                   // const vector<const MateHit*>& hits = known.mate_hits();
+                   // bool contains_spliced_hit = false;
+                   // foreach (const MateHit* h, hits)
+                   // {
+                   //     const ReadHit* left = h->left_alignment();
+                   //     const ReadHit* right = h->right_alignment();
+                   //     if (left && left->contains_splice())
+                   //     {
+                   //         contains_spliced_hit = true;
+                   //         break;
+                   //     }
+                   //     if (right && right->contains_splice())
+                   //     {
+                   //         contains_spliced_hit = true;
+                   //         break;
+                   //     }
+				   // }
+                    
+                   // if (!hits.empty() && !contains_spliced_hit && !known.is_ref())
+                   // {
+                   //     known.strand(CUFF_STRAND_UNKNOWN);
+                   // }
+                    
                     complete.push_back(known);
                 }
                     
@@ -1288,10 +1658,129 @@ void Scaffold::get_complete_subscaffolds(vector<Scaffold>& complete)
                 leftmost_known_op = -1;
 			}
 		}
-		
 	}
 }
 
+double Scaffold::internal_exon_coverage() const
+{
+    // First check if there are internal exons
+    if (augmented_ops().size() < 5) 
+        return 0.0;
+    
+    int left = augmented_ops()[2].g_left();
+    int right = augmented_ops()[augmented_ops().size() - 3].g_right();
+    vector<bool> covered(right-left, 0);
+    foreach(const MateHit* h, mate_hits())
+    {
+        if (::overlap_in_genome(h->left(),h->right(), left, right))
+        {
+            for (int i = max(h->left()-left, 0); i < min(h->right()-left, right-left); ++i)
+            {
+                assert(i < covered.size());
+                covered[i] = 1;
+            }
+        }
+    }
+    double percent_covered = accumulate(covered.begin(),covered.end(),0.0)/(right-left);
+    return percent_covered;
+}
+
+bool Scaffold::has_strand_support(vector<shared_ptr<Scaffold> >* ref_scaffs) const
+{
+	if (strand() == CUFF_STRAND_UNKNOWN)
+		return false;
+	
+	// FIXME:  This is not true for non-canonical splicing
+	if (has_intron())
+		return true;
+	
+	foreach (const MateHit* h, mate_hits())
+	{
+		if (h->strand() == strand())
+			return true;
+		assert(h->strand() == CUFF_STRAND_UNKNOWN && !h->contains_splice());
+	}
+	
+	if (ref_scaffs == NULL)
+		return false;
+	
+	foreach (shared_ptr<Scaffold const> ref_scaff, *ref_scaffs)
+	{
+		if (ref_scaff->strand() == strand() && exons_overlap(*this, *ref_scaff))
+			return true;
+	}
+		
+	return false;
+}
+
+bool Scaffold::has_struct_support(set<AugmentedCuffOp>& hit_introns) const
+{
+    if(augmented_ops().size() == 1)
+        return mate_hits().size() > 0;
+
+    if(augmented_ops().size() == 3)
+        return hits_support_introns(hit_introns);
+    
+    return (hits_support_introns(hit_introns) && internal_exon_coverage() == 1.0);
+}
+
+
+bool Scaffold::hits_support_introns() const
+{
+    set<AugmentedCuffOp> hit_introns;
+    set<AugmentedCuffOp> scaffold_introns;
+    foreach(const MateHit* h, _mates_in_scaff)
+    {
+        Scaffold s(*h);
+        foreach (AugmentedCuffOp a, s.augmented_ops())
+        {
+            if (a.opcode == CUFF_INTRON)
+            {
+                hit_introns.insert(a);
+            }
+        }
+    }
+    foreach (AugmentedCuffOp a, _augmented_ops)
+    {
+        if (a.opcode == CUFF_INTRON)
+        {
+            scaffold_introns.insert(a);
+        }
+    }
+    
+    if (hit_introns != scaffold_introns)
+    {
+        fprintf(stderr, "********************\n");
+        foreach(const AugmentedCuffOp& a, hit_introns)
+        {
+            fprintf(stderr, "%d - %d\n", a.g_left(), a.g_right());
+        }
+        
+        fprintf(stderr, "####################\n");    
+        foreach(const AugmentedCuffOp& a, scaffold_introns)
+        {
+            fprintf(stderr, "%d - %d\n", a.g_left(), a.g_right());
+        }
+    }
+    
+    return hit_introns == scaffold_introns;
+}
+
+bool Scaffold::hits_support_introns(set<AugmentedCuffOp>& hit_introns) const
+{
+    set<AugmentedCuffOp> scaffold_introns;
+
+    foreach (AugmentedCuffOp a, _augmented_ops)
+    {
+        if (a.opcode == CUFF_INTRON)
+        {
+            scaffold_introns.insert(a);
+        }
+    }
+    
+    return includes(hit_introns.begin(),hit_introns.end(), scaffold_introns.begin(), scaffold_introns.end());
+}
+
 bool scaff_lt(const Scaffold& lhs, const Scaffold& rhs)
 {
 	return lhs.left() < rhs.left();
