@@ -35,7 +35,18 @@ struct AugmentedCuffOp
 		assert (genomic_length >= 0);
 	}
 	
+	void g_left(int left) 
+	{ 
+		int right = genomic_offset + genomic_length;
+		genomic_offset = left;
+		genomic_length = right - left;
+	}
 	int g_left() const { return genomic_offset; }
+	
+	void g_right(int right) 
+	{ 
+		genomic_length = right - genomic_offset;
+	}
 	int g_right() const { return genomic_offset + genomic_length; }
 	
 	static bool overlap_in_genome(const AugmentedCuffOp& lhs,
@@ -95,7 +106,8 @@ struct AugmentedCuffOp
 	}
 	
 	static bool compatible(const AugmentedCuffOp& lhs,
-						   const AugmentedCuffOp& rhs);
+						   const AugmentedCuffOp& rhs,
+						   int overhang_tolerance = bowtie_overhang_tolerance);
 	
 	bool operator==(const AugmentedCuffOp& rhs) const
 	{
@@ -131,11 +143,13 @@ struct AugmentedCuffOp
     
     static void merge_ops(const std::vector<AugmentedCuffOp>& ops, 
 						  vector<AugmentedCuffOp>& merged,
-						  bool introns_overwrite_matches);
+						  bool introns_overwrite_matches,
+                          bool allow_flank_introns = false);
     
     static void fill_interstices(vector<AugmentedCuffOp>& to_fill,
 								 const vector<AugmentedCuffOp>& filler,
-								 bool allow_flank_fill);
+								 bool allow_flank_fill,
+                                 bool allow_flank_introns);
 	
 	CuffOpCode opcode;
 	int genomic_offset;
@@ -172,7 +186,8 @@ class Scaffold
                     //g_left -= cig[i].length;
 					break;
                 case DEL:
-                    ops.back().genomic_length += cig[i].length;
+					if (!ops.empty())
+						ops.back().genomic_length += cig[i].length;
                     g_left += cig[i].length;
 					break;
 				default:
@@ -189,14 +204,15 @@ public:
 	
 	Scaffold() :
 		_ref_id(0), 
+		_is_ref(false),
 		_strand(CUFF_STRAND_UNKNOWN), 
-		_classcode(0) {}
+		_classcode(0),
+        _fpkm(0.0) {}
 	
 	Scaffold(const MateHit& mate) :
-		  _ref_id(mate.ref_id()),
-		  _left(mate.left()), 
-	      _right(mate.right()),
-	      _classcode(0)
+		_ref_id(mate.ref_id()),
+		_is_ref(false),
+	    _classcode(0)
 	{
 		const ReadHit* left_hit = mate.left_alignment();
 		//CuffAlign a;
@@ -243,66 +259,82 @@ public:
 		}
 		_mates_in_scaff.push_back(&mate);
 		sort(aug_ops.begin(), aug_ops.end(), AugmentedCuffOp::g_left_lt);
-        
-        foreach(AugmentedCuffOp& op, aug_ops)
+
+        for(size_t i = 0; i < aug_ops.size(); ++i)
         {
-            assert (op.genomic_length >= 1);
+            assert (aug_ops[i].genomic_length >= 1);
         }
         
         AugmentedCuffOp::merge_ops(aug_ops, _augmented_ops, false);
-        
-		_right = _augmented_ops.back().g_right();
-		_left = _augmented_ops.front().g_left();
 		
-		
-		int r_check = _left;
+		int r_check = left();
 		for (size_t i = 0; i < _augmented_ops.size(); ++i)
 			r_check += _augmented_ops[i].genomic_length;
 		
 #ifdef DEBUG
-		if (r_check != _right)
+		if (r_check != right())
 		{
             AugmentedCuffOp::merge_ops(aug_ops, _augmented_ops, false);
 		}
 #endif
-		assert (r_check == _right);
+		assert (r_check == right());
 		
 		_has_intron = has_intron(*this);
 		
-		//FIXME: reinstate this assert after fixing the Graveley bug.
-		assert(!_has_intron || _strand != CUFF_STRAND_UNKNOWN);
+		assert(!has_strand_support() || _strand != CUFF_STRAND_UNKNOWN);
 
-        assert (_augmented_ops.front().opcode == CUFF_MATCH);
+        for(size_t i = 1; i < _augmented_ops.size(); ++i)
+		{
+			assert(_augmented_ops[i-1].g_right() == _augmented_ops[i].g_left());
+		}
+		
+		assert (_augmented_ops.front().opcode == CUFF_MATCH);
         assert (_augmented_ops.back().opcode == CUFF_MATCH);
+        
+        if (library_type == "transfrags")
+        {
+            double avg_fpkm = mate.mass();
+            fpkm(avg_fpkm);
+        }
 	}
 	
 	Scaffold(const vector<Scaffold>& hits, bool introns_overwrite_matches = true) 
-		:  _classcode(0)
+		:  _is_ref(false), _classcode(0)
 	{
 		assert (!hits.empty());
 		_ref_id = hits[0].ref_id();
 		
 		Scaffold::merge(hits, *this, introns_overwrite_matches);
 		
-		assert(!_has_intron || _strand != CUFF_STRAND_UNKNOWN);
+		assert(!has_strand_support() || _strand != CUFF_STRAND_UNKNOWN);
         
         assert (_augmented_ops.front().opcode == CUFF_MATCH);
         assert (_augmented_ops.back().opcode == CUFF_MATCH);
+        
+        if (library_type == "transfrags")
+        {
+            double avg_fpkm = 0.0;
+            foreach (const Scaffold& sc, hits)
+            {
+                avg_fpkm += sc.fpkm();
+            }
+            avg_fpkm /= hits.size();
+            fpkm(avg_fpkm);
+        }
 	}
 	
 	// For manually constructing scaffolds, for example when a reference is 
 	// available
-	Scaffold(RefID ref_id, CuffStrand strand, const vector<AugmentedCuffOp>& ops)
+	Scaffold(RefID ref_id, CuffStrand strand, const vector<AugmentedCuffOp>& ops, bool is_ref = false)
 	: _ref_id(ref_id), 
 	  _augmented_ops(ops), 
 	  _strand(strand),
 	  _classcode(0)
 	{
-		_right = _augmented_ops.back().g_right();
-		_left = _augmented_ops.front().g_left();
 		_has_intron = has_intron(*this);
+		_is_ref = is_ref;
 		
-		assert(!_has_intron || _strand != CUFF_STRAND_UNKNOWN);
+		assert(!has_strand_support() || _strand != CUFF_STRAND_UNKNOWN);
 
         assert (_augmented_ops.front().opcode == CUFF_MATCH);
         assert (_augmented_ops.back().opcode == CUFF_MATCH);
@@ -310,9 +342,13 @@ public:
 	
 	//int get_id() const { return _id; }
 		
-	int left() const { return _left; }
+	int left() const { return _augmented_ops.front().g_left(); }
+	int right() const  { return _augmented_ops.back().g_right(); }
 	
-	int right() const  { return _right; }
+	const vector<const MateHit*>& mate_hits() const { return _mates_in_scaff; }
+	
+	RefID ref_id() const { return _ref_id; }
+	void ref_id(RefID rid) { _ref_id = rid; }
 	
 	const string& annotated_trans_id() const { return _annotated_trans_id; }
 	void annotated_trans_id(const string& ann_name) { _annotated_trans_id = ann_name; }
@@ -353,23 +389,51 @@ public:
 		return -1.0;
 	}
 	
-			
 	char nearest_ref_classcode() const { return _classcode; }
 	void nearest_ref_classcode(char cc) { _classcode = cc; }
 	
 	bool has_intron() const { return _has_intron; }
 	bool has_suspicious_unknown() const { return has_suspicious_unknown(*this); }
+
+    // returns the fraction coverage of internal exons, returns 0 if no internal exons
+	double internal_exon_coverage() const;
+    
+    // returns true if the scaffold strand is supported with reads or exon overlap with
+    // a reference scaffold of known strand (since the scaffold may have been created with faux reads)
+    bool has_strand_support(vector<shared_ptr<Scaffold> >* ref_scaffs = NULL) const;
+    
+    // returns true if all introns are supported with splice reads, false ow
+    bool hits_support_introns() const; 
+    bool hits_support_introns(set<AugmentedCuffOp>& hit_introns) const; 
+
+    // returns true if all internal exons are fully covered and hits support introns, false ow
+    bool has_struct_support(set<AugmentedCuffOp>& hit_introns) const;
+    
+	bool is_ref() const { return _is_ref; }
+	void is_ref(bool ir) { _is_ref = ir; }
 	
-	CuffStrand strand() const { return _strand; }
-	void strand(CuffStrand strand) { _strand = strand; }
+	CuffStrand strand() const 
+    { 
+        return _strand; 
+    }
+    
+	void strand(CuffStrand strand) 
+    { 
+		assert(!has_strand_support() || _strand != CUFF_STRAND_UNKNOWN);
+        _strand = strand; 
+    }
 	
 	// Could we merge lhs and rhs together?
 	static bool compatible(const Scaffold& lhs, 
-						   const Scaffold& rhs);
+						   const Scaffold& rhs,
+						   int overhang_tolerance = bowtie_overhang_tolerance);
 	
 	static bool strand_agree(const Scaffold& lhs, 
 							 const Scaffold& rhs);
 	
+	static bool exons_overlap(const Scaffold& lhs,
+							  const Scaffold& rhs);
+	
 	// Incorporate Scaffold chow into this one.
 	static void merge(const Scaffold& lhs, 
 					  const Scaffold& rhs, 
@@ -380,12 +444,71 @@ public:
 					  Scaffold& merged,
 					  bool introns_overwrite_matches);
 
-	const vector<const MateHit*>& mate_hits() const { return _mates_in_scaff; }
-	RefID ref_id() const { return _ref_id; }
-	
-	bool contains(const Scaffold& other) const
+	// Extend 5' end using beginning of other scaffold without adding new exons.
+	void extend_5(const Scaffold& other);
+	// Clip final 3' exon by given amount
+	void trim_3(int to_remove);
+    
+    // Extend final 3' exon by given amount
+    void extend_3(int to_add);
+
+	void tile_with_scaffs(vector<Scaffold>& tile_scaffs, int max_len, int tile_offset) const;
+
+	// Creates a scaffold that matches this one but only covers the section from g_left for
+	// a distance of match_length.  It is assumed that this region is contained in the scaffold.
+	// sub_scaff should be an empty Scaffold object.
+	bool sub_scaffold(Scaffold& sub_scaff, int g_left, int match_length) const;
+
+ 	// Tests whether the other scaffold is contained allowing the given overhang
+	bool contains(const Scaffold& other, int ohang_5 = 0, int ohang_3 = 0) const
 	{
-		return (left() <= other.left() && right() >= other.right());
+		if (left() <= other.left() && right()>= other.right())
+			return true;
+		
+        if (!(ohang_5 || ohang_3))
+            return false;
+        
+		int left_hang;
+		int right_hang;
+		switch(strand())
+		{
+			case CUFF_FWD:
+				left_hang = ohang_5;
+				right_hang = ohang_3;
+				break;
+			case CUFF_REV:
+				left_hang = ohang_3;
+				right_hang = ohang_5;
+				break;
+			default:
+				left_hang = max(ohang_3, ohang_5);
+				right_hang = left_hang;
+		}
+				
+		// Test to see if it is contained within the relaxed boundaries
+		if ((left()-left_hang) <= other.left() && (right() + right_hang) >= other.right())
+		{
+			// Ensure that there are no exons outside of the strict boundaries
+			return (other.augmented_ops().front().g_right() > left() && other.augmented_ops().back().g_left() < right()); 
+		}
+		
+		return false;
+
+	}
+	
+	// Tests whether the other scaffold contains the 5' end and is contained (allowing some overhang) on the 3' end
+	// There can be no additional exons on the 5' end
+	bool overlapped_3(const Scaffold& other, int ohang_5 = 0, int ohang_3 = 0) const
+	{
+		switch(strand())
+		{
+			case CUFF_FWD:
+				return((left() + ohang_5 >= other.left() && right() + ohang_3 >= other.right()) && other.augmented_ops().front().g_right() > left());
+			case CUFF_REV:
+				return ((right() - ohang_5 <= other.right() && left() - ohang_3 <= other.left()) && other.augmented_ops().back().g_left() < right());
+			default:
+				return false;
+		}
 	}
 	
 	int match_length(int left, int right) const;
@@ -431,6 +554,8 @@ public:
 						  const AugmentedCuffOp& rhs);
 	
 	const vector<AugmentedCuffOp>& augmented_ops() const { return _augmented_ops; }
+	void augmented_ops(vector<AugmentedCuffOp> aug_ops) { _augmented_ops = aug_ops; }
+
 	
 	static bool overlap_in_genome(const Scaffold& lhs, 
 								  const Scaffold& rhs, 
@@ -472,7 +597,6 @@ public:
 	bool add_hit(const MateHit*);
 	
 	void get_complete_subscaffolds(vector<Scaffold>& complete);
-    
 private: 
 	
 	void initialize_exon_lists();
@@ -514,17 +638,10 @@ private:
 	static double min_score(const Scaffold& contig, 
 							const vector<pair<int, int> >& inners);
 	
-	static bool distance_compatible_contigs(const Scaffold& lhs, 
-											const Scaffold& rhs);
-	
 	static bool compatible_contigs(const Scaffold& lhs, 
-								   const Scaffold& rhs)
-	{
-		
-		bool dist = distance_compatible_contigs(lhs,
-												rhs);
-		return (dist);
-	}
+											const Scaffold& rhs,
+											int overhang_tolerance = bowtie_overhang_tolerance);
+	
 	
 	typedef vector<AugmentedCuffOp> OpList;
 	
@@ -540,9 +657,8 @@ private:
 	
 	vector<const MateHit*> _mates_in_scaff;
 	
-	int _left;
-	int _right;
 	bool _has_intron; 
+	bool _is_ref;
 	
 	vector<AugmentedCuffOp> _augmented_ops;
 	CuffStrand _strand;
